
define('splunk/charting/Legend',['require','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var ChainedEvent = require("jgatt").events.ChainedEvent;
	var Event = require("jgatt").events.Event;
	var EventData = require("jgatt").events.EventData;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var ArrayUtils = require("jgatt").utils.ArrayUtils;
	var Dictionary = require("jgatt").utils.Dictionary;

	return jg_extend(Object, function(Legend, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);

		// Public Events

		this.settingLabels = new Event("settingLabels", EventData);
		this.labelIndexMapChanged = new ChainedEvent("labelIndexMapChanged", this.changed);

		// Public Properties

		this.labels = new ObservableProperty("labels", Array, [])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			})
			.onChanged(function(e)
			{
				this._updateLabelMap();
			});

		// Private Properties

		this._targetMap = null;
		this._targetList = null;
		this._labelMap = null;
		this._labelList = null;
		this._isSettingLabels = false;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._targetMap = new Dictionary();
			this._targetList = [];
			this._labelMap = {};
			this._labelList = [];
		};

		// Public Methods

		this.register = function(target)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");

			var targetData = this._targetMap.get(target);
			if (targetData)
				return;

			targetData = { labels: null };
			this._targetMap.set(target, targetData);
			this._targetList.push(targetData);
		};

		this.unregister = function(target)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");

			var targetData = this._targetMap.get(target);
			if (!targetData)
				return;

			var targetIndex = ArrayUtils.indexOf(this._targetList, targetData);
			if (targetIndex >= 0)
				this._targetList.splice(targetIndex, 1);
			this._targetMap.del(target);

			this._updateLabelMap();
		};

		this.setLabels = function(target, labels)
		{
			if (target == null)
				throw new Error("Parameter target must be non-null.");
			if ((labels != null) && !(labels instanceof Array))
				throw new Error("Parameter labels must be an array.");

			var targetData = this._targetMap.get(target);
			if (!targetData)
				return;

			targetData.labels = labels ? labels.concat() : null;

			this.notifySettingLabels();
		};

		this.getLabelIndex = function(label)
		{
			if (label == null)
				throw new Error("Parameter label must be non-null.");
			if (typeof label !== "string")
				throw new Error("Parameter label must be a string.");

			var index = this.getLabelIndexOverride(label);
			if (index < 0)
			{
				var labelIndex = this._labelMap[label];
				index = (labelIndex != null) ? labelIndex : -1;
			}
			return index;
		};

		this.getNumLabels = function()
		{
			var value = this.getNumLabelsOverride();
			if (value < 0)
				value = this._labelList.length;
			return value;
		};

		this.notifySettingLabels = function()
		{
			if (this._isSettingLabels)
				return;

			try
			{
				this._isSettingLabels = true;
				this.dispatchEvent(this.settingLabels, new EventData());
				this._updateLabelMap();
			}
			finally
			{
				this._isSettingLabels = false;
			}
		};

		this.notifyLabelIndexMapChanged = function()
		{
			this.dispatchEvent(this.labelIndexMapChanged, new EventData());
		};

		// Protected Methods

		this.getNumLabelsOverride = function()
		{
			return -1;
		};

		this.getLabelIndexOverride = function(label)
		{
			return -1;
		};

		this.updateLabelsOverride = function(labels)
		{
			return false;
		};

		// Private Methods

		this._updateLabelMap = function()
		{
			var currentLabelList = this._labelList;
			var changed = false;

			var labelMap = {};
			var labelList = [];

			var targetList = this._targetList;
			var targetData;
			var targetLabels;
			var targetLabel;

			var i;
			var j;
			var l;
			var m;

			targetLabels = this.getInternal(this.labels);
			for (i = 0, l = targetLabels.length; i < l; i++)
			{
				targetLabel = String(targetLabels[i]);
				if (labelMap[targetLabel] == null)
				{
					labelMap[targetLabel] = labelList.length;
					labelList.push(targetLabel);
				}
			}

			for (i = 0, l = targetList.length; i < l; i++)
			{
				targetData = targetList[i];
				targetLabels = targetData.labels;
				if (targetLabels)
				{
					for (j = 0, m = targetLabels.length; j < m; j++)
					{
						targetLabel = String(targetLabels[j]);
						if (labelMap[targetLabel] == null)
						{
							labelMap[targetLabel] = labelList.length;
							labelList.push(targetLabel);
						}
					}
				}
			}

			if (labelList.length != currentLabelList.length)
			{
				changed = true;
			}
			else
			{
				for (i = 0, l = labelList.length; i < l; i++)
				{
					if (labelList[i] !== currentLabelList[i])
					{
						changed = true;
						break;
					}
				}
			}

			if (changed)
			{
				this._labelMap = labelMap;
				this._labelList = labelList;

				if (!this.updateLabelsOverride(labelList.concat()))
					this.notifyLabelIndexMapChanged();
			}
		};

	});

});

define('splunk/charting/ExternalLegend',['require','splunk.legend','jg_global','jgatt','splunk/charting/Legend'],function(require)
{

	var SplunkLegend = require("splunk.legend");
	var jg_extend = require("jg_global").jg_extend;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var Legend = require("splunk/charting/Legend");

	return jg_extend(Legend, function(ExternalLegend, base)
	{

		// Private Static Properties

		var _instanceCount = 0;

		// Private Properties

		this._id = null;
		this._isConnected = false;
		this._cachedExternalNumLabels = -1;
		this._cachedExternalLabelMap = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._external_setLabels = FunctionUtils.bind(this._external_setLabels, this);
			this._external_labelIndexMapChanged = FunctionUtils.bind(this._external_labelIndexMapChanged, this);

			this._id = "splunk-charting-ExternalLegend-" + (++_instanceCount);
		};

		// Public Methods

		this.connect = function()
		{
			this.close();

			SplunkLegend.register(this._id);
			SplunkLegend.addEventListener("setLabels", this._external_setLabels);
			SplunkLegend.addEventListener("labelIndexMapChanged", this._external_labelIndexMapChanged);

			this._isConnected = true;
		};

		this.close = function()
		{
			if (!this._isConnected)
				return;

			this._isConnected = false;

			SplunkLegend.removeEventListener("labelIndexMapChanged", this._external_labelIndexMapChanged);
			SplunkLegend.removeEventListener("setLabels", this._external_setLabels);
			SplunkLegend.unregister(this._id);
		};

		this.isConnected = function()
		{
			return this._isConnected;
		};

		// Protected Methods

		this.getNumLabelsOverride = function()
		{
			if (this._isConnected)
			{
				var value = this._cachedExternalNumLabels;
				if (value < 0)
					value = this._cachedExternalNumLabels = SplunkLegend.numLabels();
				return value;
			}

			return -1;
		};

		this.getLabelIndexOverride = function(label)
		{
			if (this._isConnected)
			{
				var labelMap = this._cachedExternalLabelMap;
				if (!labelMap)
					labelMap = this._cachedExternalLabelMap = {};
				var index = labelMap[label];
				if (index == null)
					index = labelMap[label] = SplunkLegend.getLabelIndex(label);
				return index;
			}

			return -1;
		};

		this.updateLabelsOverride = function(labels)
		{
			if (this._isConnected)
			{
				this._cachedExternalNumLabels = -1;
				this._cachedExternalLabelMap = null;
				SplunkLegend.setLabels(this._id, labels);
				return true;
			}

			return false;
		};

		// Private Methods

		this._external_setLabels = function()
		{
			this.notifySettingLabels();
		};

		this._external_labelIndexMapChanged = function()
		{
			this._cachedExternalNumLabels = -1;
			this._cachedExternalLabelMap = null;

			this.notifyLabelIndexMapChanged();
		};

	});

});

define('contrib/text!contrib/leaflet/leaflet.css',[],function () { return '/* required styles */\n\n.leaflet-map-pane,\n.leaflet-tile,\n.leaflet-marker-icon,\n.leaflet-marker-shadow,\n.leaflet-tile-pane,\n.leaflet-overlay-pane,\n.leaflet-shadow-pane,\n.leaflet-marker-pane,\n.leaflet-popup-pane,\n.leaflet-overlay-pane svg,\n.leaflet-zoom-box,\n.leaflet-image-layer,\n.leaflet-layer { /* TODO optimize classes */\n\tposition: absolute;\n\t}\n.leaflet-container {\n\toverflow: hidden;\n\toutline: 0;\n\t}\n.leaflet-tile,\n.leaflet-marker-icon,\n.leaflet-marker-shadow {\n\t-moz-user-select: none;\n\t-webkit-user-select: none;\n\tuser-select: none;\n\t}\n.leaflet-marker-icon,\n.leaflet-marker-shadow {\n\tdisplay: block;\n\t}\n.leaflet-clickable {\n\tcursor: pointer;\n\t}\n.leaflet-dragging, .leaflet-dragging .leaflet-clickable {\n\tcursor: move;\n\t}\n.leaflet-container img {\n    /* map is broken in FF if you have max-width: 100% on tiles */\n\tmax-width: none !important;\n\t}\n.leaflet-container img.leaflet-image-layer {\n    /* stupid Android 2 doesn\'t understand "max-width: none" properly */\n    max-width: 15000px !important;\n    }\n\n.leaflet-tile-pane { z-index: 2; }\n.leaflet-objects-pane { z-index: 3; }\n.leaflet-overlay-pane { z-index: 4; }\n.leaflet-shadow-pane { z-index: 5; }\n.leaflet-marker-pane { z-index: 6; }\n.leaflet-popup-pane { z-index: 7; }\n\n.leaflet-tile {\n    filter: inherit;\n    visibility: hidden;\n\t}\n.leaflet-tile-loaded {\n\tvisibility: inherit;\n\t}\n\n.leaflet-zoom-box {\n    width: 0;\n    height: 0;\n    }\n\n/* Leaflet controls */\n\n.leaflet-control {\n\tposition: relative;\n\tz-index: 7;\n\tpointer-events: auto;\n\t}\n.leaflet-top,\n.leaflet-bottom {\n\tposition: absolute;\n\tz-index: 1000;\n\tpointer-events: none;\n\t}\n.leaflet-top {\n\ttop: 0;\n\t}\n.leaflet-right {\n\tright: 0;\n\t}\n.leaflet-bottom {\n\tbottom: 0;\n\t}\n.leaflet-left {\n\tleft: 0;\n\t}\n.leaflet-control {\n\tfloat: left;\n\tclear: both;\n\t}\n.leaflet-right .leaflet-control {\n\tfloat: right;\n\t}\n.leaflet-top .leaflet-control {\n\tmargin-top: 10px;\n\t}\n.leaflet-bottom .leaflet-control {\n\tmargin-bottom: 10px;\n\t}\n.leaflet-left .leaflet-control {\n\tmargin-left: 10px;\n\t}\n.leaflet-right .leaflet-control {\n\tmargin-right: 10px;\n\t}\n\n.leaflet-control-zoom {\n\t-moz-border-radius: 7px;\n\t-webkit-border-radius: 7px;\n\tborder-radius: 7px;\n\t}\n.leaflet-control-zoom {\n\tpadding: 5px;\n\tbackground: rgba(0, 0, 0, 0.25);\n\t}\n.leaflet-control-zoom a {\n\tbackground-color: rgba(255, 255, 255, 0.75);\n\t}\n.leaflet-control-zoom a, .leaflet-control-layers a {\n\tbackground-position: 50% 50%;\n\tbackground-repeat: no-repeat;\n\tdisplay: block;\n\t}\n.leaflet-control-zoom a {\n\t-moz-border-radius: 4px;\n\t-webkit-border-radius: 4px;\n\tborder-radius: 4px;\n\twidth: 19px;\n\theight: 19px;\n\t}\n.leaflet-control-zoom a:hover {\n\tbackground-color: #fff;\n\t}\n.leaflet-touch .leaflet-control-zoom a {\n\twidth: 27px;\n\theight: 27px;\n\t}\n.leaflet-control-zoom-in {\n\tbackground-image: url(images/zoom-in.png);\n\tmargin-bottom: 5px;\n\t}\n.leaflet-control-zoom-out {\n\tbackground-image: url(images/zoom-out.png);\n\t}\n\n.leaflet-control-layers {\n\tbox-shadow: 0 1px 7px #999;\n\tbackground: #f8f8f9;\n\t-moz-border-radius: 8px;\n\t-webkit-border-radius: 8px;\n\tborder-radius: 8px;\n\t}\n.leaflet-control-layers a {\n\tbackground-image: url(images/layers.png);\n\twidth: 36px;\n\theight: 36px;\n\t}\n.leaflet-touch .leaflet-control-layers a {\n\twidth: 44px;\n\theight: 44px;\n\t}\n.leaflet-control-layers .leaflet-control-layers-list,\n.leaflet-control-layers-expanded .leaflet-control-layers-toggle {\n\tdisplay: none;\n\t}\n.leaflet-control-layers-expanded .leaflet-control-layers-list {\n\tdisplay: block;\n\tposition: relative;\n\t}\n.leaflet-control-layers-expanded {\n\tpadding: 6px 10px 6px 6px;\n\tfont: 12px/1.5 "Helvetica Neue", Arial, Helvetica, sans-serif;\n\tcolor: #333;\n\tbackground: #fff;\n\t}\n.leaflet-control-layers input {\n\tmargin-top: 2px;\n\tposition: relative;\n\ttop: 1px;\n\t}\n.leaflet-control-layers label {\n\tdisplay: block;\n\t}\n.leaflet-control-layers-separator {\n\theight: 0;\n\tborder-top: 1px solid #ddd;\n\tmargin: 5px -10px 5px -6px;\n\t}\n\n.leaflet-container .leaflet-control-attribution {\n\tbackground-color: rgba(255, 255, 255, 0.7);\n\tbox-shadow: 0 0 5px #bbb;\n\tmargin: 0;\n    }\n\n.leaflet-control-attribution,\n.leaflet-control-scale-line {\n\tpadding: 0 5px;\n\tcolor: #333;\n\t}\n\n.leaflet-container .leaflet-control-attribution,\n.leaflet-container .leaflet-control-scale {\n\tfont: 11px/1.5 "Helvetica Neue", Arial, Helvetica, sans-serif;\n\t}\n\n.leaflet-left .leaflet-control-scale {\n\tmargin-left: 5px;\n\t}\n.leaflet-bottom .leaflet-control-scale {\n\tmargin-bottom: 5px;\n\t}\n\n.leaflet-control-scale-line {\n\tborder: 2px solid #777;\n\tborder-top: none;\n\tcolor: black;\n\tline-height: 1;\n\tfont-size: 10px;\n\tpadding-bottom: 2px;\n\ttext-shadow: 1px 1px 1px #fff;\n\tbackground-color: rgba(255, 255, 255, 0.5);\n\t}\n.leaflet-control-scale-line:not(:first-child) {\n\tborder-top: 2px solid #777;\n\tpadding-top: 1px;\n\tborder-bottom: none;\n\tmargin-top: -2px;\n\t}\n.leaflet-control-scale-line:not(:first-child):not(:last-child) {\n\tborder-bottom: 2px solid #777;\n\t}\n\n.leaflet-touch .leaflet-control-attribution, .leaflet-touch .leaflet-control-layers {\n\tbox-shadow: none;\n\t}\n.leaflet-touch .leaflet-control-layers {\n\tborder: 5px solid #bbb;\n\t}\n\n\n/* Zoom and fade animations */\n\n.leaflet-fade-anim .leaflet-tile, .leaflet-fade-anim .leaflet-popup {\n\topacity: 0;\n\n\t-webkit-transition: opacity 0.2s linear;\n\t-moz-transition: opacity 0.2s linear;\n\t-o-transition: opacity 0.2s linear;\n\ttransition: opacity 0.2s linear;\n\t}\n.leaflet-fade-anim .leaflet-tile-loaded, .leaflet-fade-anim .leaflet-map-pane .leaflet-popup {\n\topacity: 1;\n\t}\n\n.leaflet-zoom-anim .leaflet-zoom-animated {\n\t-webkit-transition: -webkit-transform 0.25s cubic-bezier(0.25,0.1,0.25,0.75);\n\t-moz-transition: -moz-transform 0.25s cubic-bezier(0.25,0.1,0.25,0.75);\n\t-o-transition: -o-transform 0.25s cubic-bezier(0.25,0.1,0.25,0.75);\n\ttransition: transform 0.25s cubic-bezier(0.25,0.1,0.25,0.75);\n\t}\n\n.leaflet-zoom-anim .leaflet-tile,\n.leaflet-pan-anim .leaflet-tile,\n.leaflet-touching .leaflet-zoom-animated {\n    -webkit-transition: none;\n    -moz-transition: none;\n    -o-transition: none;\n    transition: none;\n    }\n\n.leaflet-zoom-anim .leaflet-zoom-hide {\n\tvisibility: hidden;\n\t}\n\n\n/* Popup layout */\n\n.leaflet-popup {\n\tposition: absolute;\n\ttext-align: center;\n\t}\n.leaflet-popup-content-wrapper {\n\tpadding: 1px;\n\ttext-align: left;\n\t}\n.leaflet-popup-content {\n\tmargin: 14px 20px;\n\t}\n.leaflet-popup-tip-container {\n\tmargin: 0 auto;\n\twidth: 40px;\n\theight: 20px;\n\tposition: relative;\n\toverflow: hidden;\n\t}\n.leaflet-popup-tip {\n\twidth: 15px;\n\theight: 15px;\n\tpadding: 1px;\n\n\tmargin: -8px auto 0;\n\n\t-moz-transform: rotate(45deg);\n\t-webkit-transform: rotate(45deg);\n\t-ms-transform: rotate(45deg);\n\t-o-transform: rotate(45deg);\n\ttransform: rotate(45deg);\n\t}\n.leaflet-container a.leaflet-popup-close-button {\n\tposition: absolute;\n\ttop: 0;\n\tright: 0;\n\tpadding: 4px 5px 0 0;\n\ttext-align: center;\n\twidth: 18px;\n\theight: 14px;\n\tfont: 16px/14px Tahoma, Verdana, sans-serif;\n\tcolor: #c3c3c3;\n\ttext-decoration: none;\n\tfont-weight: bold;\n\t}\n.leaflet-container a.leaflet-popup-close-button:hover {\n\tcolor: #999;\n\t}\n.leaflet-popup-content p {\n\tmargin: 18px 0;\n\t}\n.leaflet-popup-scrolled {\n\toverflow: auto;\n\tborder-bottom: 1px solid #ddd;\n\tborder-top: 1px solid #ddd;\n\t}\n\n\n/* Visual appearance */\n\n.leaflet-container {\n\tbackground: #ddd;\n\t}\n.leaflet-container a {\n\tcolor: #0078A8;\n\t}\n.leaflet-container a.leaflet-active {\n    outline: 2px solid orange;\n    }\n.leaflet-zoom-box {\n\tborder: 2px dotted #05f;\n\tbackground: white;\n\topacity: 0.5;\n\t}\n.leaflet-div-icon {\n    background: #fff;\n    border: 1px solid #666;\n    }\n.leaflet-editing-icon {\n    border-radius: 2px;\n    }\n.leaflet-popup-content-wrapper, .leaflet-popup-tip {\n\tbackground: white;\n\n\tbox-shadow: 0 3px 10px #888;\n\t-moz-box-shadow: 0 3px 10px #888;\n\t-webkit-box-shadow: 0 3px 14px #999;\n\t}\n.leaflet-popup-content-wrapper {\n\t-moz-border-radius: 20px;\n\t-webkit-border-radius: 20px;\n\tborder-radius: 20px;\n\t}\n.leaflet-popup-content {\n\tfont: 12px/1.4 "Helvetica Neue", Arial, Helvetica, sans-serif;\n\t}\n';});

define('contrib/text!contrib/leaflet/leaflet.ie.css',[],function () { return '.leaflet-vml-shape {\n\twidth: 1px;\n\theight: 1px;\n\t}\n.lvml {\n\tbehavior: url(#default#VML); \n\tdisplay: inline-block; \n\tposition: absolute;\n\t}\n\t\n.leaflet-control {\n\tdisplay: inline;\n\t}\n\n.leaflet-popup-tip {\n\twidth: 21px;\n\t_width: 27px;\n\tmargin: 0 auto;\n\t_margin-top: -3px;\n\t\n\tfilter: progid:DXImageTransform.Microsoft.Matrix(M11=0.70710678, M12=0.70710678, M21=-0.70710678, M22=0.70710678);\n\t-ms-filter: "progid:DXImageTransform.Microsoft.Matrix(M11=0.70710678, M12=0.70710678, M21=-0.70710678, M22=0.70710678)";\n\t}\n.leaflet-popup-tip-container {\n\tmargin-top: -1px;\n\t}\n.leaflet-popup-content-wrapper, .leaflet-popup-tip {\n\tborder: 1px solid #bbb;\n\t}\n.leaflet-popup-content-wrapper {\n\tzoom: 1;\n\t}\n\n.leaflet-control-zoom {\n\tfilter: progid:DXImageTransform.Microsoft.gradient(startColorStr=\'#3F000000\',EndColorStr=\'#3F000000\');\n\t}\n.leaflet-control-zoom a {\n\tbackground-color: #eee;\n\t}\n.leaflet-control-zoom a:hover {\n\tbackground-color: #fff;\n\t}\n.leaflet-control-layers-toggle {\n\t}\n.leaflet-control-attribution, .leaflet-control-layers {\n\tbackground: white;\n\t}';});

define('splunk/events/GenericEventData',['require','jg_global','jgatt'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var EventData = require("jgatt").events.EventData;

	return jg_extend(EventData, function(GenericEventData, base)
	{

		// Constructor

		this.constructor = function(attributes)
		{
			if (attributes != null)
			{
				for (var a in attributes)
				{
					if (attributes.hasOwnProperty(a) && !(a in this))
						this[a] = attributes[a];
				}
			}
		};

	});

});

define('splunk/mapping/LatLon',['require','jg_global'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;

	return jg_extend(Object, function(LatLon, base)
	{

		// Public Properties

		this.lat = 0;
		this.lon = 0;

		// Constructor

		this.constructor = function(lat, lon)
		{
			this.lat = (lat !== undefined) ? lat : 0;
			this.lon = (lon !== undefined) ? lon : 0;
		};

		// Public Methods

		this.normalize = function(center)
		{
			var lat = this.lat;
			if (lat < -90)
				lat = -90;
			else if (lat > 90)
				lat = 90;

			var centerLon = center ? center.lon : 0;
			var lon = (this.lon - centerLon) % 360;
			if (lon < -180)
				lon += 360;
			else if (lon > 180)
				lon -= 360;
			lon += centerLon;

			return new LatLon(lat, lon);
		};

		this.isFinite = function()
		{
			return (((this.lat - this.lat) === 0) &&
			        ((this.lon - this.lon) === 0));
		};

		this.equals = function(latLon)
		{
			return ((this.lat == latLon.lat) &&
			        (this.lon == latLon.lon));
		};

		this.clone = function()
		{
			return new LatLon(this.lat, this.lon);
		};

		this.toString = function()
		{
			return "(" + this.lat + ", " + this.lon + ")";
		};

	});

});

define('splunk/mapping/LatLonBounds',['require','jg_global','splunk/mapping/LatLon'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var LatLon = require("splunk/mapping/LatLon");

	return jg_extend(Object, function(LatLonBounds, base)
	{

		// Public Properties

		this.s = 0;
		this.w = 0;
		this.n = 0;
		this.e = 0;

		// Constructor

		this.constructor = function(s, w, n, e)
		{
			this.s = (s !== undefined) ? s : 0;
			this.w = (w !== undefined) ? w : 0;
			this.n = (n !== undefined) ? n : 0;
			this.e = (e !== undefined) ? e : 0;
		};

		// Public Methods

		this.getSW = function()
		{
			return new LatLon(this.s, this.w);
		};

		this.getSE = function()
		{
			return new LatLon(this.s, this.e);
		};

		this.getNW = function()
		{
			return new LatLon(this.n, this.w);
		};

		this.getNE = function()
		{
			return new LatLon(this.n, this.e);
		};

		this.getCenter = function()
		{
			return new LatLon((this.s + this.n) / 2, (this.w + this.e) / 2);
		};

		this.expand = function(latLon)
		{
			if (latLon.lat < this.s)
				this.s = latLon.lat;
			if (latLon.lat > this.n)
				this.n = latLon.lat;
			if (latLon.lon < this.w)
				this.w = latLon.lon;
			if (latLon.lon > this.e)
				this.e = latLon.lon;
		};

		this.contains = function(latLon)
		{
			return ((latLon.lat >= this.s) &&
			        (latLon.lat <= this.n) &&
			        (latLon.lon >= this.w) &&
			        (latLon.lon <= this.e));
		};

		this.normalize = function(center)
		{
			var s = this.s;
			if (s < -90)
				s = -90;
			else if (s > 90)
				s = 90;

			var n = this.n;
			if (n < s)
				n = s;
			else if (n > 90)
				n = 90;

			var centerLon = center ? center.lon : 0;
			var w = (this.w - centerLon);
			var e = (this.e - centerLon);
			if ((e - w) >= 360)
			{
				w = -180;
				e = 180;
			}
			else
			{
				w %= 360;
				if (w < -180)
					w += 360;
				else if (w > 180)
					w -= 360;

				e %= 360;
				if (e < -180)
					e += 360;
				else if (e > 180)
					e -= 360;

				if (e < w)
				{
					if (e > -w)
						w -= 360;
					else
						e += 360;
				}
			}
			w += centerLon;
			e += centerLon;

			return new LatLonBounds(s, w, n, e);
		};

		this.isFinite = function()
		{
			return (((this.s - this.s) === 0) &&
			        ((this.w - this.w) === 0) &&
			        ((this.n - this.n) === 0) &&
			        ((this.e - this.e) === 0));
		};

		this.equals = function(bounds)
		{
			return ((this.s == bounds.s) &&
			        (this.w == bounds.w) &&
			        (this.n == bounds.n) &&
			        (this.e == bounds.e));
		};

		this.clone = function()
		{
			return new LatLonBounds(this.s, this.w, this.n, this.e);
		};

		this.toString = function()
		{
			return "(" + this.s + ", " + this.w + ", " + this.n + ", " + this.e + ")";
		};

	});

});

define('splunk/viz/MRenderTarget',['require','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt','jgatt'],function(require)
{

	var jg_static = require("jg_global").jg_static;
	var jg_mixin = require("jg_global").jg_mixin;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var PropertyComparator = require("jgatt").utils.PropertyComparator;
	var MValidateTarget = require("jgatt").validation.MValidateTarget;
	var ValidatePass = require("jgatt").validation.ValidatePass;

	return jg_static(function(MRenderTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
			base = jg_mixin(this, MValidateTarget, base);
		};

		// Public Passes

		this.renderPass = new ValidatePass("render", 3, new PropertyComparator("renderPriority"));

		// Public Properties

		this.renderPriority = 0;

		// Public Methods

		this.render = function()
		{
		};

	});

});

define('splunk/mapping/layers/LayerBase',['require','jg_global','jg_global','jgatt','jgatt','splunk/viz/MRenderTarget'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var Property = require("jgatt").properties.Property;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var MRenderTarget = require("splunk/viz/MRenderTarget");

	return jg_extend(Object, function(LayerBase, base)
	{

		base = jg_mixin(this, MRenderTarget, base);

		// Public Static Constants

		LayerBase.METADATA_KEY = "__splunk_mapping_layers_LayerBase_metadata";

		// Public Properties

		this.map = new Property("map", Object, null, true);

		this.leafletLayer = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);

			this.leafletLayer = this.createLeafletLayer();
			if (!this.leafletLayer)
				throw new Error("Value returned from createLeafletLayer() must be non-null.");
		};

		// Public Methods

		this.render = function()
		{
			this.validatePreceding("renderPass");

			if (this.isValid("renderPass"))
				return;

			var map = this.getInternal("map");
			if (map)
				this.renderOverride(map);

			this.setValid("renderPass");
		};

		// Protected Methods

		this.createLeafletLayer = function()
		{
			throw new Error("Must implement method createLeafletLayer.");
		};

		this.renderOverride = function(map)
		{
		};

		this.onAddedToMap = function(map)
		{
			this.setInternal("map", map);

			map.addEventListener("boundsChanged", this._map_boundsChanged);

			this.invalidate("renderPass");
		};

		this.onRemovedFromMap = function(map)
		{
			map.removeEventListener("boundsChanged", this._map_boundsChanged);

			this.setInternal("map", null);
		};

		// Private Methods

		this._map_boundsChanged = function(e)
		{
			this.invalidate("renderPass");
		};

	});

});

define('splunk/viz/VizBase',['require','jquery','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt'],function(require)
{

	var $ = require("jquery");
	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var Property = require("jgatt").properties.Property;
	var MValidateTarget = require("jgatt").validation.MValidateTarget;

	return jg_extend(Object, function(VizBase, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);
		base = jg_mixin(this, MValidateTarget, base);

		// Private Static Constants

		var _INSTANCE_KEY = "__splunk_viz_VizBase_instance";

		// Private Static Properties

		var _instanceCount = 0;

		// Public Static Methods

		VizBase.getInstance = function(element)
		{
			if (element == null)
				return null;

			element = $(element);
			if (element.length == 0)
				return null;

			element = element[0];

			var instance = element[_INSTANCE_KEY];
			return (instance instanceof VizBase) ? instance : null;
		};

		// Public Properties

		this.id = new Property("id", String, null, true);

		this.element = null;
		this.$element = null;

		// Constructor

		this.constructor = function(html)
		{
			if ((html != null) && (typeof html !== "string"))
				throw new Error("Parameter html must be a string.");

			var query = $(html ? html : "<div></div>");
			if (query.length == 0)
				throw new Error("Parameter html must be valid markup.");

			base.constructor.call(this);

			var id = "splunk-viz-VizBase-" + (++_instanceCount);

			this.element = query[0];
			//this.element[_INSTANCE_KEY] = this;
			//this.element.id = id;

			this.$element = $(this.element);

			this.setInternal("id", id);

			this.addStyleClass("splunk-viz-VizBase");
		};

		// Public Methods

		this.addStyleClass = function(styleClass)
		{
			this.$element.addClass(styleClass);
		};

		this.removeStyleClass = function(styleClass)
		{
			this.$element.removeClass(styleClass);
		};

		this.setStyle = function(style)
		{
			this.$element.css(style);
		};

		this.appendTo = function(parentElement)
		{
			if (parentElement == null)
				throw new Error("Parameter parentElement must be non-null.");

			if (parentElement instanceof VizBase)
				parentElement = parentElement.element;

			parentElement = $(parentElement);
			if (parentElement.length == 0)
				return;

			parentElement = parentElement[0];

			var oldParent = this.element.parentNode;
			if (oldParent && (oldParent !== parentElement))
				this.onRemove();

			parentElement.appendChild(this.element);

			if (oldParent !== parentElement)
				this.onAppend();
		};

		this.replace = function(element)
		{
			if (element == null)
				throw new Error("Parameter element must be non-null.");

			if (element instanceof VizBase)
				element = element.element;

			element = $(element);
			if (element.length == 0)
				return;

			element = element[0];

			var parentElement = element.parentNode;
			if (parentElement == null)
				return;

			var oldParent = this.element.parentNode;
			if (oldParent && (oldParent !== parentElement))
				this.onRemove();

			parentElement.replaceChild(this.element, element);

			if (oldParent !== parentElement)
				this.onAppend();
		};

		this.remove = function()
		{
			var element = this.element;
			var parentElement = element.parentNode;
			if (!parentElement)
				return;

			this.onRemove();

			parentElement.removeChild(element);
		};

		this.dispose = function()
		{
			this.remove();

			// ensure all jquery data and events are removed
			this.$element.remove();
		};

		// Protected Methods

		this.onAppend = function()
		{
		};

		this.onRemove = function()
		{
		};

	});

});

define('splunk/mapping/Map',['require','leaflet','jg_global','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','splunk/events/GenericEventData','splunk/mapping/LatLon','splunk/mapping/LatLonBounds','splunk/mapping/layers/LayerBase','splunk/viz/VizBase'],function(require)
{

	var Leaflet = require("leaflet");
	var jg_extend = require("jg_global").jg_extend;
	var ChainedEvent = require("jgatt").events.ChainedEvent;
	var Event = require("jgatt").events.Event;
	var EventData = require("jgatt").events.EventData;
	var ObservableArrayProperty = require("jgatt").properties.ObservableArrayProperty;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var Property = require("jgatt").properties.Property;
	var ArrayUtils = require("jgatt").utils.ArrayUtils;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var NumberUtils = require("jgatt").utils.NumberUtils;
	var StringUtils = require("jgatt").utils.StringUtils;
	var ValidatePass = require("jgatt").validation.ValidatePass;
	var GenericEventData = require("splunk/events/GenericEventData");
	var LatLon = require("splunk/mapping/LatLon");
	var LatLonBounds = require("splunk/mapping/LatLonBounds");
	var LayerBase = require("splunk/mapping/layers/LayerBase");
	var VizBase = require("splunk/viz/VizBase");

	var Map = jg_extend(VizBase, function(Map, base)
	{

		// Private Static Constants

		var _MIN_LAT = -85.051128779806;
		var _MAX_LAT =  85.051128779806;

		// Public Passes

		this.updateLeafletMapSizePass = new ValidatePass("updateLeafletMapSize", -1);
		this.updateTilesPass = new ValidatePass("updateTiles", 0);

		// Public Events

		this.boundsChanged = new ChainedEvent("boundsChanged", this.changed, EventData);
		this.mapClicked = new Event("mapClicked", GenericEventData);

		// Public Properties

		this.center = new Property("center", LatLon, null)
			.getter(function()
			{
				var center = this.leafletMap.getCenter();
				return new LatLon(center.lat, center.lng);
			})
			.setter(function(value)
			{
				value = (value && value.isFinite()) ? value : new LatLon();

				this.validate();
				this.leafletMap.setView(new Leaflet.LatLng(value.lat, value.lon), this.leafletMap.getZoom(), true);

				this._checkBoundsChanged();

				// set a second time on a delay since Leaflet is a POS and doesn't set the
				// center properly if zoom, minZoom, or maxZoom are also set at the same time
				clearTimeout(this._setCenterTimeout);
				this._setCenterTimeout = setTimeout(FunctionUtils.bind(function()
				{
					this.leafletMap.setView(new Leaflet.LatLng(value.lat, value.lon), this.leafletMap.getZoom(), true);
					this._checkBoundsChanged();
				}, this), 250);
			});

		this.zoom = new Property("zoom", Number, 0)
			.getter(function()
			{
				return this.leafletMap.getZoom();
			})
			.setter(function(value)
			{
				value = ((value >= 0) && (value < Infinity)) ? value : 0;

				this.validate();
				this.leafletMap.setView(this.leafletMap.getCenter(), value, true);

				this._checkBoundsChanged();
			});

		this.tileURL = new ObservableProperty("tileURL", String, null)
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.tileSubdomains = new ObservableArrayProperty("tileSubdomains", String, [ "a", "b", "c" ])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			})
			.changedComparator(function(oldValue, newValue)
			{
				if (oldValue.length !== newValue.length)
					return true;

				for (var i = 0, l = oldValue.length; i < l; i++)
				{
					if (oldValue[i] !== newValue[i])
						return true;
				}

				return false;
			})
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.tileMinZoom = new ObservableProperty("tileMinZoom", Number, 0)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.tileMaxZoom = new ObservableProperty("tileMaxZoom", Number, Infinity)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? Math.floor(value) : Infinity;
			})
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.tileInvertY = new ObservableProperty("tileInvertY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.tileAttribution = new ObservableProperty("tileAttribution", String, null)
			.onChanged(function(e)
			{
				this.invalidate("updateTilesPass");
			});

		this.leafletMap = null;

		this.formatNumber = null;
		this.formatDegrees = null;

		// Private Properties

		this._tooltip = null;
		this._tooltipMetadata = null;
		this._layers = null;
		this._tileLayer = null;
		this._width = 0;
		this._height = 0;
		this._bounds = null;
		this._updateSizeInterval = 0;
		this._setCenterTimeout = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addStyleClass("splunk-mapping-Map");

			this.setStyle({ position: "relative", width: "100%", height: "100%", overflow: "hidden", background: "none" });

			this.updateSize = FunctionUtils.bind(this.updateSize, this);
			this._leafletMap_moveend = FunctionUtils.bind(this._leafletMap_moveend, this);
			this._leafletMap_zoomend = FunctionUtils.bind(this._leafletMap_zoomend, this);
			this._self_mouseOver = FunctionUtils.bind(this._self_mouseOver, this);
			this._self_mouseOut = FunctionUtils.bind(this._self_mouseOut, this);
			this._self_mouseMove = FunctionUtils.bind(this._self_mouseMove, this);
			this._self_click = FunctionUtils.bind(this._self_click, this);

			this.leafletMap = new Leaflet.Map(this.element, { center: new Leaflet.LatLng(0, 0), zoom: 0, trackResize: false, worldCopyJump: false });
			this.leafletMap.attributionControl.setPrefix("");
			this.leafletMap.on("moveend", this._leafletMap_moveend);
			this.leafletMap.on("zoomend", this._leafletMap_zoomend);

			this._tooltip = new LeafletTooltip();

			this._layers = [];

			this.$element.bind("mouseover", this._self_mouseOver);
			this.$element.bind("mouseout", this._self_mouseOut);
			this.$element.bind("mousemove", this._self_mouseMove);
			this.$element.bind("click", this._self_click);
		};

		// Public Methods

		this.updateLeafletMapSize = function()
		{
			this.validatePreceding("updateLeafletMapSizePass");

			if (this.isValid("updateLeafletMapSizePass"))
				return;

			this.leafletMap.invalidateSize();
			// hack to force immediate redraw
			clearTimeout(this.leafletMap._sizeTimer);
			this.leafletMap.fire("moveend");

			this.setValid("updateLeafletMapSizePass");
		};

		this.updateTiles = function()
		{
			this.validatePreceding("updateTilesPass");

			if (this.isValid("updateTilesPass"))
				return;

			var leafletMap = this.leafletMap;

			var tileLayer = this._tileLayer;
			if (tileLayer)
			{
				leafletMap.removeLayer(tileLayer);
				this._tileLayer = null;
			}

			var tileOptions = {};
			tileOptions.subdomains = this.getInternal("tileSubdomains");
			tileOptions.minZoom = this.getInternal("tileMinZoom");
			tileOptions.maxZoom = this.getInternal("tileMaxZoom");
			tileOptions.tms = this.getInternal("tileInvertY");
			tileOptions.attribution = this.getInternal("tileAttribution");

			var tileURL = this.getInternal("tileURL");
			if (tileURL)
			{
				tileLayer = this._tileLayer = new Leaflet.TileLayer(tileURL, tileOptions);
				leafletMap.addLayer(tileLayer, true);
			}

			// hack to adjust maxZoom on leafletMap
			leafletMap.options.minZoom = tileOptions.minZoom;
			leafletMap.options.maxZoom = tileOptions.maxZoom;
			leafletMap.setZoom(leafletMap.getZoom());

			this.setValid("updateTilesPass");

			this._checkBoundsChanged();
		};

		this.addLayer = function(layer)
		{
			if (layer == null)
				throw new Error("Parameter layer must be non-null.");
			if (!(layer instanceof LayerBase))
				throw new Error("Parameter layer must be an instance of splunk.mapping.layers.LayerBase");

			var layers = this._layers;
			if (ArrayUtils.indexOf(layers, layer) >= 0)
				return;

			layers.push(layer);
			this.leafletMap.addLayer(layer.leafletLayer);
			layer.onAddedToMap(this);
		};

		this.removeLayer = function(layer)
		{
			if (layer == null)
				throw new Error("Parameter layer must be non-null.");
			if (!(layer instanceof LayerBase))
				throw new Error("Parameter layer must be an instance of splunk.mapping.layers.LayerBase");

			var layers = this._layers;
			var index = ArrayUtils.indexOf(layers, layer);
			if (index < 0)
				return;

			layer.onRemovedFromMap(this);
			this.leafletMap.removeLayer(layer.leafletLayer);
			layers.splice(index, 1);
		};

		this.fitWorld = function(viewportInside)
		{
			if ((viewportInside != null) && (typeof viewportInside !== "boolean"))
				throw new Error("Parameter viewportInside must be a boolean.");

			this.fitBounds(new LatLonBounds(-60, -180, 85, 180), viewportInside);
		};

		this.fitBounds = function(latLonBounds, viewportInside)
		{
			if (latLonBounds == null)
				throw new Error("Parameter latLonBounds must be non-null.");
			if (!(latLonBounds instanceof LatLonBounds))
				throw new Error("Parameter latLonBounds must be an instance of splunk.mapping.LatLonBounds.");
			if ((viewportInside != null) && (typeof viewportInside !== "boolean"))
				throw new Error("Parameter viewportInside must be a boolean.");

			latLonBounds = latLonBounds.isFinite() ? latLonBounds : new LatLonBounds(-60, -180, 85, 180);
			viewportInside = (viewportInside === true);

			var bounds = new Leaflet.LatLngBounds(new Leaflet.LatLng(latLonBounds.s, latLonBounds.w), new Leaflet.LatLng(latLonBounds.n, latLonBounds.e));

			// Leaflet's bounds.getCenter() doesn't handle non-normalized bounds
			// so have to use ours and convert to Leaflet's
			var center = latLonBounds.getCenter();
			center = new Leaflet.LatLng(center.lat, center.lon);

			var zoom = this.leafletMap.getBoundsZoom(bounds, viewportInside);

			this.leafletMap.setView(center, zoom, true);

			this._checkBoundsChanged();
		};

		this.getLatLonBounds = function()
		{
			var bounds = this.leafletMap.getBounds();
			var sw = bounds.getSouthWest();
			var ne = bounds.getNorthEast();
			return new LatLonBounds(sw.lat, sw.lng, ne.lat, ne.lng);
		};

		this.updateSize = function()
		{
			var width = this.$element.width();
			var height = this.$element.height();
			if ((width === this._width) && (height === this._height))
				return;

			this._width = width;
			this._height = height;

			this.leafletMap.invalidateSize();
			this.invalidate("updateLeafletMapSizePass");

			this._checkBoundsChanged();
		};

		this.dispose = function()
		{
			clearTimeout(this._setCenterTimeout);

			var layers = this._layers.concat();
			for (var i = layers.length - 1; i >= 0; i--)
				this.removeLayer(layers[i]);

			base.dispose.call(this);
		};

		// Protected Methods

		this.onAppend = function()
		{
			this._updateSizeInterval = setInterval(this.updateSize, 50);

			this.updateSize();
		};

		this.onRemove = function()
		{
			clearInterval(this._updateSizeInterval);
		};

		// Private Methods

		this._checkBoundsChanged = function()
		{
			var oldBounds = this._bounds;
			var newBounds = this.getLatLonBounds();
			if (oldBounds && oldBounds.equals(newBounds))
				return;

			this._bounds = newBounds;

			this.dispatchEvent("boundsChanged", new EventData());
		};

		this._updateTooltip = function(element)
		{
			var tooltip = this._tooltip;
			var metadata = this._getMetadataFromElement(element);

			if (metadata && (metadata !== this._tooltipMetadata))
			{
				this._tooltipMetadata = metadata;

				var data = metadata.data;
				var fields = metadata.fields;
				var sliceList = metadata.sliceList;
				var tooltipLatLng = metadata.tooltipLatLng;
				var tooltipOffsetRadius = metadata.tooltipOffsetRadius;

				if (data && fields && tooltipLatLng)
				{
					var content = "";
					var field;
					var slice;
					var i, l;

					content += "<table style=\"border: 0 none; border-spacing: 0; border-collapse: collapse;\">";
					for (i = 0, l = Math.min(fields.length, 2); i < l; i++)
					{
						field = fields[i];
						content += "<tr>";
						content += "<td style=\"padding: 0; text-align: left; white-space: nowrap; color: #333333;\">" + StringUtils.escapeHTML(field) + ":&nbsp;&nbsp;</td><td style=\"padding: 0; text-align: right; white-space: nowrap;\">" + StringUtils.escapeHTML(this._formatDegrees(data[field], (i === 0) ? "ns" : "ew")) + "</td>";
						content += "</tr>";
					}
					for (i = 0, l = sliceList.length; i < l; i++)
					{
						slice = sliceList[i];
						content += "<tr>";
						content += "<td style=\"padding: 0; text-align: left; white-space: nowrap; color: " + ("#" + (slice.series.color | 0x1000000).toString(16).substring(1)) + ";\">" + StringUtils.escapeHTML(slice.series.name) + ":&nbsp;&nbsp;</td><td style=\"padding: 0; text-align: right; white-space: nowrap;\">" + StringUtils.escapeHTML(this._formatNumber(slice.value)) + "</td>";
						content += "</tr>";
					}
					content += "</table>";

					tooltip.setLatLng(tooltipLatLng);
					tooltip.setOffsetRadius(tooltipOffsetRadius);
					tooltip.setContent(content);

					this.leafletMap.openPopup(tooltip);
				}
				else
				{
					this.leafletMap.closePopup();
				}
			}
			else if (!metadata && this._tooltipMetadata)
			{
				this._tooltipMetadata = null;

				this.leafletMap.closePopup();
			}
		};

		this._getMetadataFromElement = function(element)
		{
			while (element)
			{
				if (element[LayerBase.METADATA_KEY])
					return element[LayerBase.METADATA_KEY];
				element = element.parentNode;
			}
			return null;
		};

		this._formatNumber = function(num)
		{
			var format = this.formatNumber;
			if (typeof format === "function")
				return format(Number(num));

			return String(num);
		};

		this._formatDegrees = function(degrees, orientation)
		{
			var format = this.formatDegrees;
			if (typeof format === "function")
				return format(Number(degrees), orientation);

			return String(degrees);
		};

		this._leafletMap_moveend = function(e)
		{
			this._checkBoundsChanged();
		};

		this._leafletMap_zoomend = function(e)
		{
			this._checkBoundsChanged();
		};

		this._self_mouseOver = function(e)
		{
			this._updateTooltip(e.target);
		};

		this._self_mouseOut = function(e)
		{
			this._updateTooltip(e.target);
		};

		this._self_mouseMove = function(e)
		{
			this._updateTooltip(e.target);
		};

		this._self_click = function(e)
		{
			if (this.leafletMap.dragging && this.leafletMap.dragging.moved())
				return;

			var metadata = this._getMetadataFromElement(e.target);
			if (!metadata || !metadata.data || !metadata.fields)
				return;

			e.preventDefault();

			var data = {};
			for (var p in metadata.data)
				data[p] = metadata.data[p];
			var fields = metadata.fields.concat();

			this.dispatchEvent("mapClicked", new GenericEventData({ data: data, fields: fields, altKey: e.altKey, ctrlKey: e.ctrlKey || e.metaKey, shiftKey: e.shiftKey, jQueryEvent: e, originalEvent: e.originalEvent }));
		};

	});

	var LeafletTooltip = Leaflet.Popup.extend({

		options: {
			paddingX: 5,
			paddingY: 5
		},

		_offsetRadius: 0,

		initialize: function(options) {
			options = Leaflet.Util.extend(options || {}, { maxWidth: Infinity, maxHeight: Infinity, autoPan: false, closeButton: false });
			Leaflet.Popup.prototype.initialize.call(this, options);
		},

		setOffsetRadius: function(offsetRadius) {
			this._offsetRadius = offsetRadius;
			this._update();
			return this;
		},

		_initLayout: function() {
			Leaflet.Popup.prototype._initLayout.call(this);

			// hide tip
			this._tipContainer.style.display = "none";
		},

		_updatePosition: function() {
			var map = this._map;
			var mapTL = map.containerPointToLayerPoint(new Leaflet.Point(0, 0));
			var mapBR = map.containerPointToLayerPoint(map.getSize());
			var mapLeft = mapTL.x;
			var mapTop = mapTL.y;
			var mapRight = mapBR.x;
			var mapBottom = mapBR.y;

			var container = this._container;
			var containerWidth = container.offsetWidth;
			var containerHeight = container.offsetHeight;

			var is3d = L.Browser.any3d;
			var offsetRadius = this._offsetRadius;
			var paddingX = this.options.paddingX;
			var paddingY = this.options.paddingY;

			var centerPoint = map.latLngToLayerPoint(this._latlng);
			var offsetX = (centerPoint.x > ((mapLeft + mapRight) / 2)) ? (-containerWidth - offsetRadius - paddingX) : offsetRadius + paddingX;
			var offsetY = NumberUtils.maxMin(centerPoint.y - containerHeight / 2, mapBottom - containerHeight - paddingY, mapTop + paddingY) - centerPoint.y;

			if (is3d)
				L.DomUtil.setPosition(container, centerPoint);

			var x = offsetX + (is3d ? 0 : centerPoint.x);
			var y = offsetY + (is3d ? 0 : centerPoint.y);

			container.style.left = Math.round(x) + "px";
			container.style.top = Math.round(y) + "px";
		}

	});

	// override Leaflet.Control.Attribution so that the attribution container is hidden when there is no text
	Leaflet.Control.Attribution.include({

		_update: function () {
			if (!this._map) { return; }

			var attribs = [];

			for (var i in this._attributions) {
				if (this._attributions.hasOwnProperty(i) && this._attributions[i]) {
					attribs.push(i);
				}
			}

			var prefixAndAttribs = [];

			if (this.options.prefix) {
				prefixAndAttribs.push(this.options.prefix);
			}
			if (attribs.length) {
				prefixAndAttribs.push(attribs.join(', '));
			}

			var text = prefixAndAttribs.join(' &#8212; ');

			this._container.innerHTML = text;
			this._container.style.display = text ? "" : "none";
		}

	});

	return Map;

});

define('splunk/vectors/VectorElement',['require','jg_global','jg_global','jg_global'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var jg_static = require("jg_global").jg_static;

	return jg_extend(Object, function(VectorElement, base)
	{

		// Private Static Constants

		var _HAS_SVG = (typeof document.createElementNS === "function");
		var _HAS_VML = (!_HAS_SVG && (function()
		{
			try
			{
				document.namespaces.add("splvml", "urn:schemas-microsoft-com:vml");

				var styleText = ".splvml { behavior: url(#default#VML); display: inline-block; position: absolute; }";

				var styleNode = document.createElement("style");
				styleNode.setAttribute("type", "text/css");

				var headNode = document.getElementsByTagName("head")[0];
				headNode.appendChild(styleNode);

				if (styleNode.styleSheet)
					styleNode.styleSheet.cssText = styleText;
				else
					styleNode.appendChild(document.createTextNode(styleText));

				return true;
			}
			catch (e)
			{
				return false;
			}
		})());

		// Public Static Methods

		VectorElement.mixin = function(target, sourceSVG, sourceVML)
		{
			if (_HAS_SVG)
			{
				jg_mixin(target, sourceSVG);
				// jg_mixin doesn't copy constructor, so do it manually
				if ((sourceSVG.constructor !== Object) && (typeof sourceSVG.constructor === "function"))
					target.constructor = sourceSVG.constructor;
			}
			else if (_HAS_VML)
			{
				jg_mixin(target, sourceVML);
				// jg_mixin doesn't copy constructor, so do it manually
				if ((sourceVML.constructor !== Object) && (typeof sourceVML.constructor === "function"))
					target.constructor = sourceVML.constructor;
			}
		};

		// Public Properties

		this.hasSVG = _HAS_SVG;
		this.hasVML = _HAS_VML;
		this.element = null;

		// Constructor

		this.constructor = function(tagName)
		{
			if ((tagName != null) && (typeof tagName !== "string"))
				throw new Error("Parameter tagName must be a string.");

			this.element = this.createElement(tagName || null);
		};

		// Public Methods

		this.appendTo = function(parentElement)
		{
			if (parentElement == null)
				throw new Error("Parameter parentElement must be non-null.");
			if (!(parentElement instanceof VectorElement))
				throw new Error("Parameter parentElement must be an instance of splunk.vectors.VectorElement.");

			parentElement.element.appendChild(this.element);

			return this;
		};

		this.remove = function()
		{
			if (this.element.parentNode)
				this.element.parentNode.removeChild(this.element);

			return this;
		};

		this.dispose = function()
		{
			this.remove();

			this.element = null;
		};

		this.display = function(value)
		{
			this.element.style.display = value ? value : "";

			return this;
		};

		this.visibility = function(value)
		{
			this.element.style.visibility = value ? value : "";

			return this;
		};

		this.translate = function(x, y)
		{
			x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
			y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

			this.element.style.left = (x != 0) ? x + "px" : "";
			this.element.style.top = (y != 0) ? y + "px" : "";

			return this;
		};

		// Protected Methods

		this.createElement = function(tagName)
		{
			var dummy = document.createElement("div");
			dummy.style.position = "absolute";
			return dummy;
		};

		// Inner Mixin Classes

		var SVGVectorElement = jg_static(function(SVGVectorElement)
		{

			// Private Static Constants

			var _NS_SVG = "http://www.w3.org/2000/svg";

			// Public Methods

			this.display = function(value)
			{
				if (value)
					this.element.setAttribute("display", value);
				else
					this.element.removeAttribute("display");

				return this;
			};

			this.visibility = function(value)
			{
				if (value)
					this.element.setAttribute("visibility", value);
				else
					this.element.removeAttribute("visibility");

				return this;
			};

			this.translate = function(x, y)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

				if ((x != 0) || (y != 0))
					this.element.setAttribute("transform", "translate(" + x + "," + y + ")");
				else
					this.element.removeAttribute("transform");

				return this;
			};

			// Protected Methods

			this.createElement = function(tagName)
			{
				return document.createElementNS(_NS_SVG, tagName || "g");
			};

		});

		var VMLVectorElement = jg_static(function(VMLVectorElement)
		{

			// Protected Methods

			this.createElement = function(tagName)
			{
				return document.createElement("<splvml:" + (tagName || "group") + " class=\"splvml\">");
			};

		});

		VectorElement.mixin(this, SVGVectorElement, VMLVectorElement);

	});

});

define('splunk/vectors/Group',['require','jg_global','jg_global','splunk/vectors/VectorElement'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_static = require("jg_global").jg_static;
	var VectorElement = require("splunk/vectors/VectorElement");

	return jg_extend(VectorElement, function(Group, base)
	{

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

		// Inner Mixin Classes

		var SVGGroup = jg_static(function(SVGGroup)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "g");
			};

		});

		var VMLGroup = jg_static(function(VMLGroup)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "group");

				this.element.style.width = "1px";
				this.element.style.height = "1px";
				this.element.coordsize = "1,1";
			};

		});

		VectorElement.mixin(this, SVGGroup, VMLGroup);

	});

});

define('splunk/vectors/VectorUtils',['require','jg_global'],function(require)
{

	var jg_static = require("jg_global").jg_static;

	return jg_static(function(VectorUtils)
	{

		// Public Static Methods

		VectorUtils.toSVGString = function(element)
		{
			// svg elements don't have innerHTML attribute...
			// clone svg element and place in container div so we can use innerHTML of the container
			var clonedElement = element.cloneNode(true);
			var containerElement = document.createElement("div");
			containerElement.appendChild(clonedElement);

			// get svg string using innerHTML
			var svgString = containerElement.innerHTML;

			// fix or add xlink namespace on href attributes
			svgString = svgString.replace(/xlink:href=|href=/g, "x:href=");

			// properly close image tags
			svgString = svgString.replace(/<image([\S\s]*?)\s*\/?>\s*(<\/image>)?/g, "<image$1></image>");

			// add xmlns attributes to root svg tag
			svgString = svgString.replace(/^<svg/, "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:x=\"http://www.w3.org/1999/xlink\"");

			// clear element references
			clonedElement = null;
			containerElement = null;

			return svgString;
		};

		VectorUtils.concatSVGStrings = function(/*...*/)
		{
			var concatString = "";
			var svgString;
			var viewBoxMatch;
			var viewBox;
			var width = 0;
			var height = 0;

			for (var i = 0, l = arguments.length; i < l; i++)
			{
				svgString = arguments[i];

				// read and parse viewBox attribute from root svg tag
				viewBoxMatch = svgString.match(/^<svg[^>]*viewBox=\"([^ ]+) ([^ ]+) ([^ ]+) ([^\"]+)\"[^>]*>/);
				if (viewBoxMatch && (viewBoxMatch.length == 5))
				{
					viewBox = {
						x: Number(viewBoxMatch[1]),
						y: Number(viewBoxMatch[2]),
						width: Number(viewBoxMatch[3]),
						height: Number(viewBoxMatch[4])
					};

					// expand width and height to include viewBox
					width = Math.max(width, viewBox.width);
					height = Math.max(height, viewBox.height);
				}
				else
				{
					viewBox = null;
				}

				// replace root svg tag with g tag, including translate transform if needed
				if (viewBox && ((viewBox.x != 0) || (viewBox.y != 0)))
					svgString = svgString.replace(/^<svg[^>]*>/, "<g transform=\"translate(" + (-viewBox.x) + ", " + (-viewBox.y) + ")\">");
				else
					svgString = svgString.replace(/^<svg[^>]*>/, "<g>");
				svgString = svgString.replace(/<\/svg>$/, "</g>");

				concatString += svgString;
			}

			// generate new root svg tag around concatString
			svgString = "<svg";
			svgString += " xmlns=\"http://www.w3.org/2000/svg\"";
			svgString += " xmlns:x=\"http://www.w3.org/1999/xlink\"";
			svgString += " width=\"" + width + "\"";
			svgString += " height=\"" + height + "\"";
			svgString += " viewBox=\"0 0 " + width + " " + height + "\"";
			svgString += ">";
			svgString += concatString;
			svgString += "</svg>";

			return svgString;
		};

	});

});

define('splunk/vectors/Viewport',['require','jg_global','jg_global','jgatt','splunk/vectors/VectorElement','splunk/vectors/VectorUtils'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_static = require("jg_global").jg_static;
	var Rectangle = require("jgatt").geom.Rectangle;
	var VectorElement = require("splunk/vectors/VectorElement");
	var VectorUtils = require("splunk/vectors/VectorUtils");

	return jg_extend(VectorElement, function(Viewport, base)
	{

		// Constructor

		this.constructor = function(width, height, viewBox, preserveAspectRatio)
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.width = function(value)
		{
			return this;
		};

		this.height = function(value)
		{
			return this;
		};

		this.viewBox = function(value)
		{
			return this;
		};

		this.preserveAspectRatio = function(value)
		{
			return this;
		};

		this.toSVGString = function()
		{
			return "";
		};

		// Inner Mixin Classes

		var SVGViewport = jg_static(function(SVGViewport)
		{

			// Constructor

			this.constructor = function(width, height, viewBox, preserveAspectRatio)
			{
				base.constructor.call(this, "svg");

				this.width((width != null) ? width : 0);
				this.height((height != null) ? height : 0);
				if (viewBox != null)
					this.viewBox(viewBox);
				if (preserveAspectRatio != null)
					this.preserveAspectRatio(preserveAspectRatio);
			};

			// Public Methods

			this.appendTo = function(parentElement)
			{
				if (parentElement == null)
					throw new Error("Parameter parentElement must be non-null.");
				if (parentElement.appendChild == null)
					throw new Error("Parameter parentElement must be a DOM node.");

				parentElement.appendChild(this.element);

				return this;
			};

			this.width = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("width", Math.max(value, 0));
				else
					this.element.setAttribute("width", 0);

				return this;
			};

			this.height = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("height", Math.max(value, 0));
				else
					this.element.setAttribute("height", 0);

				return this;
			};

			this.viewBox = function(value)
			{
				if (value && (value instanceof Rectangle) && value.isFinite())
					this.element.setAttribute("viewBox", value.x + " " + value.y + " " + value.width + " " + value.height);
				else
					this.element.removeAttribute("viewBox");

				return this;
			};

			this.preserveAspectRatio = function(value)
			{
				if (value)
					this.element.setAttribute("preserveAspectRatio", value);
				else
					this.element.removeAttribute("preserveAspectRatio");

				return this;
			};

			this.toSVGString = function()
			{
				return VectorUtils.toSVGString(this.element);
			};

		});

		var VMLViewport = jg_static(function(VMLViewport)
		{

			// Private Properties

			this._containerElement = null;
			this._width = 0;
			this._height = 0;
			this._viewBox = null;

			// Constructor

			this.constructor = function(width, height, viewBox, preserveAspectRatio)
			{
				base.constructor.call(this, "group");

				this._containerElement = document.createElement("div");
				this._containerElement.style.position = "relative";
				this._containerElement.style.overflow = "hidden";
				this._containerElement.appendChild(this.element);

				this.width((width != null) ? width : 0);
				this.height((height != null) ? height : 0);
				if (viewBox != null)
					this.viewBox(viewBox);
				if (preserveAspectRatio != null)
					this.preserveAspectRatio(preserveAspectRatio);
			};

			// Public Methods

			this.appendTo = function(parentElement)
			{
				if (parentElement == null)
					throw new Error("Parameter parentElement must be non-null.");
				if (parentElement.appendChild == null)
					throw new Error("Parameter parentElement must be a DOM node.");

				parentElement.appendChild(this._containerElement);

				return this;
			};

			this.remove = function()
			{
				if (this._containerElement.parentNode)
					this._containerElement.parentNode.removeChild(this._containerElement);

				return this;
			};

			this.dispose = function()
			{
				base.dispose.call(this);

				this._containerElement = null;
			};

			this.display = function(value)
			{
				this._containerElement.style.display = value ? value : "";

				return this;
			};

			this.visibility = function(value)
			{
				this._containerElement.style.visibility = value ? value : "";

				return this;
			};

			this.translate = function(x, y)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;

				this._containerElement.style.left = (x != 0) ? x + "px" : "";
				this._containerElement.style.top = (y != 0) ? y + "px" : "";

				return this;
			};

			this.width = function(value)
			{
				this._width = ((value != null) && (value < Infinity)) ? Math.max(value, 0) : 0;
				this._updateView();

				return this;
			};

			this.height = function(value)
			{
				this._height = ((value != null) && (value < Infinity)) ? Math.max(value, 0) : 0;
				this._updateView();

				return this;
			};

			this.viewBox = function(value)
			{
				this._viewBox = (value && (value instanceof Rectangle) && value.isFinite()) ? value.clone() : null;
				this._updateView();

				return this;
			};

			this.preserveAspectRatio = function(value)
			{
				return this;
			};

			// Private Methods

			this._updateView = function()
			{
				var width = Math.round(this._width);
				var height = Math.round(this._height);
				var viewBox = this._viewBox;
				var viewX = viewBox ? Math.round(viewBox.x) : 0;
				var viewY = viewBox ? Math.round(viewBox.y) : 0;
				var viewWidth = viewBox ? Math.round(Math.max(viewBox.width, 1)) : width;
				var viewHeight = viewBox ? Math.round(Math.max(viewBox.height, 1)) : height;

				var element = this.element;
				var style = element.style;
				var containerStyle = this._containerElement.style;

				style.display = "none";  // prevent premature rendering

				element.coordorigin = viewX + "," + viewY;
				element.coordsize = viewWidth + "," + viewHeight;

				style.width = width + "px";
				style.height = height + "px";

				containerStyle.width = width + "px";
				containerStyle.height = height + "px";

				style.display = "";  // enable rendering
			};

		});

		VectorElement.mixin(this, SVGViewport, VMLViewport);

	});

});

define('splunk/mapping/layers/VectorLayerBase',['require','leaflet','jg_global','jgatt','jgatt','splunk/mapping/layers/LayerBase','splunk/vectors/Group','splunk/vectors/Viewport'],function(require)
{

	var Leaflet = require("leaflet");
	var jg_extend = require("jg_global").jg_extend;
	var Rectangle = require("jgatt").geom.Rectangle;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var LayerBase = require("splunk/mapping/layers/LayerBase");
	var Group = require("splunk/vectors/Group");
	var Viewport = require("splunk/vectors/Viewport");

	var VectorLayerBase = jg_extend(LayerBase, function(VectorLayerBase, base)
	{

		// Public Properties

		this.vectorContainer = null;
		this.vectorBounds = null;

		// Private Properties

		this._isZooming = false;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._leafletMap_move = FunctionUtils.bind(this._leafletMap_move, this);
			this._leafletMap_zoomstart = FunctionUtils.bind(this._leafletMap_zoomstart, this);
			this._leafletMap_zoomend = FunctionUtils.bind(this._leafletMap_zoomend, this);

			this.vectorContainer = this.leafletLayer.vectorContainer;
			this.vectorBounds = { minX: 0, minY: 0, maxX: 0, maxY: 0 };
		};

		// Protected Methods

		this.createLeafletLayer = function()
		{
			return new LeafletVectorLayer();
		};

		this.renderOverride = function(map)
		{
			if (!this._isZooming)
				this.vectorContainer.display(null);
		};

		this.onAddedToMap = function(map)
		{
			base.onAddedToMap.call(this, map);

			var leafletMap = map.leafletMap;
			if (this.vectorContainer.hasSVG)
				leafletMap.on("move", this._leafletMap_move);
			else
				leafletMap.on("moveend", this._leafletMap_move);
			leafletMap.on("viewreset", this._leafletMap_move);
			leafletMap.on("zoomstart", this._leafletMap_zoomstart);
			leafletMap.on("zoomend", this._leafletMap_zoomend);

			this.vectorBounds = leafletMap._vectorLayerBounds;

			this.vectorContainer.display("none");
		};

		this.onRemovedFromMap = function(map)
		{
			var leafletMap = map.leafletMap;
			if (this.vectorContainer.hasSVG)
				leafletMap.off("move", this._leafletMap_move);
			else
				leafletMap.off("moveend", this._leafletMap_move);
			leafletMap.off("viewreset", this._leafletMap_move);
			leafletMap.off("zoomstart", this._leafletMap_zoomstart);
			leafletMap.off("zoomend", this._leafletMap_zoomend);

			this.vectorBounds = { minX: 0, minY: 0, maxX: 0, maxY: 0 };

			base.onRemovedFromMap.call(this, map);
		};

		// Private Methods

		this._leafletMap_move = function(e)
		{
			this.invalidate("renderPass");
		};

		this._leafletMap_zoomstart = function(e)
		{
			this._isZooming = true;

			this.vectorContainer.display("none");
		};

		this._leafletMap_zoomend = function(e)
		{
			this._isZooming = false;

			this.invalidate("renderPass");
		};

	});

	var LeafletVectorLayer = Leaflet.Class.extend({

		includes: [Leaflet.Mixin.Events],

		options: {
			clickable: true
		},

		vectorContainer: null,

		initialize: function (options) {
			Leaflet.Util.setOptions(this, options);

			this.vectorContainer = new Group();
		},

		onAdd: function (map) {
			this._map = map;

			map._initVectorLayerViewport();

			this.vectorContainer.appendTo(map._vectorLayerViewport);
		},

		onRemove: function (map) {
			this._map = null;

			this.vectorContainer.remove();
		}

	});

	Leaflet.Map.include({

		_initVectorLayerViewport: function () {
			if (this._vectorLayerRoot)
				return;

			var root = this._vectorLayerRoot = document.createElement("div");
			root.style.position = "absolute";
			this._panes.overlayPane.appendChild(root);

			var viewport = this._vectorLayerViewport = new Viewport();
			viewport.appendTo(root);

			this._vectorLayerBounds = { minX: 0, minY: 0, maxX: 0, maxY: 0 };

			if (viewport.hasSVG)
				this.on("move", this._updateVectorLayerBounds);
			else
				this.on("moveend", this._updateVectorLayerBounds);
			this._updateVectorLayerBounds();
		},

		_updateVectorLayerBounds: function () {
			var root = this._vectorLayerRoot,
			    viewport = this._vectorLayerViewport,
			    bounds = this._vectorLayerBounds,
			    padding = viewport.hasSVG ? 0 : 0.5,
			    size = this.getSize(),
			    panePos = Leaflet.DomUtil.getPosition(this._mapPane),
			    min = panePos.multiplyBy(-1)._subtract(size.multiplyBy(padding)),
			    max = min.add(size.multiplyBy(1 + padding * 2)),
			    width = max.x - min.x,
			    height = max.y - min.y;

			bounds.minX = min.x;
			bounds.minY = min.y;
			bounds.maxX = max.x;
			bounds.maxY = max.y;

			Leaflet.DomUtil.setPosition(root, min);
			viewport.width(width);
			viewport.height(height);
			viewport.viewBox(new Rectangle(min.x, min.y, width, height));
		}

	});

	return VectorLayerBase;

});

define('splunk/palettes/ColorPalette',['require','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var Property = require("jgatt").properties.Property;
	var PropertyEventData = require("jgatt").properties.PropertyEventData;

	return jg_extend(Object, function(ColorPalette, base)
	{

		base = jg_mixin(this, MObservable, base);
		base = jg_mixin(this, MPropertyTarget, base);

		// Private Properties

		this._properties = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.addEventListener(this.changed, this._self_changed, Infinity);
		};

		// Public Methods

		this.getColor = function(field, index, count)
		{
			if ((field != null) && (typeof field !== "string"))
				throw new Error("Parameter field must be a string.");
			if (index == null)
				throw new Error("Parameter index must be non-null.");
			if (typeof index !== "number")
				throw new Error("Parameter index must be a number.");
			if (count == null)
				throw new Error("Parameter count must be non-null.");
			if (typeof count !== "number")
				throw new Error("Parameter count must be a number.");

			if (!this._properties)
				this._properties = this._getProperties();

			return this.getColorOverride(this._properties, field, Math.floor(index), Math.floor(count));
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			return 0x000000;
		};

		// Private Methods

		this._getProperties = function()
		{
			var properties = {};
			var property;
			for (var p in this)
			{
				property = this[p];
				if (property instanceof Property)
					properties[p] = this.getInternal(property);
			}
			return properties;
		};

		this._self_changed = function(e)
		{
			if ((e.target === this) && (e instanceof PropertyEventData))
				this._properties = null;
		};

	});

});

define('splunk/palettes/ListColorPalette',['require','jg_global','jgatt','jgatt','jgatt','splunk/palettes/ColorPalette'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var ColorUtils = require("jgatt").graphics.ColorUtils;
	var ObservableArrayProperty = require("jgatt").properties.ObservableArrayProperty;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var ColorPalette = require("splunk/palettes/ColorPalette");

	return jg_extend(ColorPalette, function(ListColorPalette, base)
	{

		// Public Properties

		this.colors = new ObservableArrayProperty("colors", Number, [])
			.readFilter(function(value)
			{
				return value.concat();
			})
			.writeFilter(function(value)
			{
				return value ? value.concat() : [];
			});

		this.interpolate = new ObservableProperty("interpolate", Boolean, false);

		// Constructor

		this.constructor = function(colors, interpolate)
		{
			base.constructor.call(this);

			if (colors != null)
				this.set(this.colors, colors);
			if (interpolate != null)
				this.set(this.interpolate, interpolate);
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			var colors = properties.colors;
			var numColors = colors.length;

			if (numColors == 0)
				return 0x000000;

			if (index < 0)
				index = 0;

			if (properties.interpolate)
			{
				if (count < 1)
					count = 1;
				if (index > count)
					index = count;

				var p = (count == 1) ? 0 : (numColors - 1) * (index / (count - 1));
				var index1 = Math.floor(p);
				var index2 = Math.min(index1 + 1, numColors - 1);
				p -= index1;

				return ColorUtils.interpolate(colors[index1], colors[index2], p);
			}

			return colors[index % numColors];
		};

	});

});

define('splunk/vectors/Shape',['require','jg_global','jg_global','jgatt','splunk/vectors/VectorElement'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_static = require("jg_global").jg_static;
	var NumberUtils = require("jgatt").utils.NumberUtils;
	var VectorElement = require("splunk/vectors/VectorElement");

	return jg_extend(VectorElement, function(Shape, base)
	{

		// Constructor

		this.constructor = function(tagName)
		{
			base.constructor.call(this, tagName);
		};

		// Public Methods

		this.fillColor = function(value)
		{
			return this;
		};

		this.fillOpacity = function(value)
		{
			return this;
		};

		this.strokeColor = function(value)
		{
			return this;
		};

		this.strokeOpacity = function(value)
		{
			return this;
		};

		this.strokeWidth = function(value)
		{
			return this;
		};

		this.strokeLineCap = function(value)
		{
			return this;
		};

		this.strokeLineJoin = function(value)
		{
			return this;
		};

		this.strokeMiterLimit = function(value)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGShape = jg_static(function(SVGShape)
		{

			// Constructor

			this.constructor = function(tagName)
			{
				base.constructor.call(this, tagName);

				this.fillColor(NaN);
				this.strokeColor(NaN);
				this.strokeLineCap("none");
				this.strokeLineJoin("miter");
			};

			// Public Methods

			this.fillColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this.element.setAttribute("fill", "#" + (value | 0x1000000).toString(16).substring(1));
				}
				else
				{
					this.element.setAttribute("fill", "none");
				}

				return this;
			};

			this.fillOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this.element.setAttribute("fill-opacity", NumberUtils.minMax(value, 0, 1));
				else
					this.element.removeAttribute("fill-opacity");

				return this;
			};

			this.strokeColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this.element.setAttribute("stroke", "#" + (value | 0x1000000).toString(16).substring(1));
				}
				else
				{
					this.element.removeAttribute("stroke");
				}

				return this;
			};

			this.strokeOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this.element.setAttribute("stroke-opacity", NumberUtils.minMax(value, 0, 1));
				else
					this.element.removeAttribute("stroke-opacity");

				return this;
			};

			this.strokeWidth = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("stroke-width", Math.max(value, 1));
				else
					this.element.removeAttribute("stroke-width");

				return this;
			};

			this.strokeLineCap = function(value)
			{
				if (value === "round")
					this.element.setAttribute("stroke-linecap", "round");
				else if (value === "square")
					this.element.setAttribute("stroke-linecap", "square");
				else  // none
					this.element.removeAttribute("stroke-linecap");

				return this;
			};

			this.strokeLineJoin = function(value)
			{
				if (value === "round")
					this.element.setAttribute("stroke-linejoin", "round");
				else if (value === "bevel")
					this.element.setAttribute("stroke-linejoin", "bevel");
				else  // miter
					this.element.removeAttribute("stroke-linejoin");

				return this;
			};

			this.strokeMiterLimit = function(value)
			{
				if ((value != null) && (value < Infinity))
					this.element.setAttribute("stroke-miterlimit", Math.max(value, 1));
				else
					this.element.removeAttribute("stroke-miterlimit");

				return this;
			};

		});

		var VMLShape = jg_static(function(VMLShape)
		{

			// Private Properties

			this._fillElement = null;
			this._strokeElement = null;

			// Constructor

			this.constructor = function(tagName)
			{
				base.constructor.call(this, tagName);

				this._fillElement = this.createElement("fill");
				this._strokeElement = this.createElement("stroke");

				this.element.appendChild(this._fillElement);
				this.element.appendChild(this._strokeElement);

				this.fillColor(NaN);
				this.strokeColor(NaN);
				this.strokeLineCap("none");
				this.strokeLineJoin("miter");
			};

			// Public Methods

			this.dispose = function()
			{
				base.dispose.call(this);

				this._fillElement = null;
				this._strokeElement = null;
			};

			this.fillColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this._fillElement.on = true;
					this._fillElement.color = "#" + (value | 0x1000000).toString(16).substring(1);
				}
				else
				{
					this._fillElement.on = false;
					this._fillElement.color = "#000000";
				}

				return this;
			};

			this.fillOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this._fillElement.opacity = NumberUtils.minMax(value, 0, 1);
				else
					this._fillElement.opacity = 1;

				return this;
			};

			this.strokeColor = function(value)
			{
				if ((value != null) && !isNaN(value))
				{
					value = NumberUtils.minMax(Math.floor(value), 0x000000, 0xFFFFFF);
					this._strokeElement.on = true;
					this._strokeElement.color = "#" + (value | 0x1000000).toString(16).substring(1);
				}
				else
				{
					this._strokeElement.on = false;
					this._strokeElement.color = "#000000";
				}

				return this;
			};

			this.strokeOpacity = function(value)
			{
				if ((value != null) && !isNaN(value))
					this._strokeElement.opacity = NumberUtils.minMax(value, 0, 1);
				else
					this._strokeElement.opacity = 1;

				return this;
			};

			this.strokeWidth = function(value)
			{
				if ((value != null) && (value < Infinity))
					this._strokeElement.weight = Math.max(value, 1) + "px";
				else
					this._strokeElement.weight = "1px";

				return this;
			};

			this.strokeLineCap = function(value)
			{
				if (value === "round")
					this._strokeElement.endcap = "round";
				else if (value === "square")
					this._strokeElement.endcap = "square";
				else // none
					this._strokeElement.endcap = "flat";

				return this;
			};

			this.strokeLineJoin = function(value)
			{
				if (value === "round")
					this._strokeElement.joinstyle = "round";
				else if (value === "bevel")
					this._strokeElement.joinstyle = "bevel";
				else // miter
					this._strokeElement.joinstyle = "miter";

				return this;
			};

			this.strokeMiterLimit = function(value)
			{
				if ((value != null) && (value < Infinity))
					this._strokeElement.miterlimit = Math.max(value, 1);
				else
					this._strokeElement.miterlimit = 4;

				return this;
			};

		});

		VectorElement.mixin(this, SVGShape, VMLShape);

	});

});

define('splunk/vectors/Wedge',['require','jg_global','jg_global','jgatt','splunk/vectors/Shape','splunk/vectors/VectorElement'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var jg_static = require("jg_global").jg_static;
	var NumberUtils = require("jgatt").utils.NumberUtils;
	var Shape = require("splunk/vectors/Shape");
	var VectorElement = require("splunk/vectors/VectorElement");

	return jg_extend(Shape, function(Wedge, base)
	{

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

		// Public Methods

		this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
		{
			return this;
		};

		// Inner Mixin Classes

		var SVGWedge = jg_static(function(SVGWedge)
		{

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "path");
			};

			// Public Methods

			this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;
				radiusX = ((radiusX != null) && (radiusX < Infinity)) ? Math.max(radiusX, 0) : 0;
				radiusY = ((radiusY != null) && (radiusY < Infinity)) ? Math.max(radiusY, 0) : 0;
				startAngle = ((startAngle != null) && (startAngle > -Infinity) && (startAngle < Infinity)) ? startAngle : 0;
				arcAngle = ((arcAngle != null) && (arcAngle != null) && !isNaN(arcAngle)) ? NumberUtils.minMax(arcAngle, -360, 360) : 0;

				if ((radiusX == 0) || (radiusY == 0) || (arcAngle == 0))
				{
					this.element.removeAttribute("d");
					return this;
				}

				var a1 = (startAngle / 180) * Math.PI;
				var x1 = x + Math.cos(a1) * radiusX;
				var y1 = y + Math.sin(a1) * radiusY;
				var a2 = ((startAngle + arcAngle / 2) / 180) * Math.PI;
				var x2 = x + Math.cos(a2) * radiusX;
				var y2 = y + Math.sin(a2) * radiusY;
				var a3 = ((startAngle + arcAngle) / 180) * Math.PI;
				var x3 = x + Math.cos(a3) * radiusX;
				var y3 = y + Math.sin(a3) * radiusY;

				var sweepFlag = (arcAngle < 0) ? 0 : 1;

				var pathData = "";
				if ((arcAngle > -360) && (arcAngle < 360))
				{
					pathData += "M" + x + "," + y;
					pathData += " L" + x1 + "," + y1;
				}
				else
				{
					pathData += "M" + x1 + "," + y1;
				}
				pathData += " A" + radiusX + "," + radiusY + " 0 0 " + sweepFlag + " " + x2 + "," + y2;
				pathData += " " + radiusX + "," + radiusY + " 0 0 " + sweepFlag + " " + x3 + "," + y3;
				pathData += " Z";

				this.element.setAttribute("d", pathData);

				return this;
			};

		});

		var VMLWedge = jg_static(function(VMLWedge)
		{

			// Private Static Constants

			var _RES = 64;

			// Private Properties

			this._pathElement = null;

			// Constructor

			this.constructor = function()
			{
				base.constructor.call(this, "shape");

				this._pathElement = this.createElement("path");

				this.element.style.width = "1px";
				this.element.style.height = "1px";
				this.element.coordsize = _RES + "," + _RES;
				this.element.appendChild(this._pathElement);
			};

			// Public Methods

			this.dispose = function()
			{
				base.dispose.call(this);

				this._pathElement = null;
			};

			this.draw = function(x, y, radiusX, radiusY, startAngle, arcAngle)
			{
				x = ((x != null) && (x > -Infinity) && (x < Infinity)) ? x : 0;
				y = ((y != null) && (y > -Infinity) && (y < Infinity)) ? y : 0;
				radiusX = ((radiusX != null) && (radiusX < Infinity)) ? Math.max(radiusX, 0) : 0;
				radiusY = ((radiusY != null) && (radiusY < Infinity)) ? Math.max(radiusY, 0) : 0;
				startAngle = ((startAngle != null) && (startAngle > -Infinity) && (startAngle < Infinity)) ? startAngle : 0;
				arcAngle = ((arcAngle != null) && (arcAngle != null) && !isNaN(arcAngle)) ? NumberUtils.minMax(arcAngle, -360, 360) : 0;

				if ((radiusX == 0) || (radiusY == 0) || (arcAngle == 0))
				{
					this._pathElement.v = " ";
					return this;
				}

				var a1 = (startAngle / 180) * Math.PI;
				var x1 = x + Math.cos(a1) * radiusX;
				var y1 = y + Math.sin(a1) * radiusY;
				var a2 = ((startAngle + arcAngle / 2) / 180) * Math.PI;
				var x2 = x + Math.cos(a2) * radiusX;
				var y2 = y + Math.sin(a2) * radiusY;
				var a3 = ((startAngle + arcAngle) / 180) * Math.PI;
				var x3 = x + Math.cos(a3) * radiusX;
				var y3 = y + Math.sin(a3) * radiusY;

				var left = Math.round((x - radiusX) * _RES);
				var top = Math.round((y - radiusY) * _RES);
				var right = Math.round((x + radiusX) * _RES);
				var bottom = Math.round((y + radiusY) * _RES);

				x = Math.round(x * _RES);
				y = Math.round(y * _RES);
				x1 = Math.round(x1 * _RES);
				y1 = Math.round(y1 * _RES);
				x2 = Math.round(x2 * _RES);
				y2 = Math.round(y2 * _RES);
				x3 = Math.round(x3 * _RES);
				y3 = Math.round(y3 * _RES);

				var pathData = "";
				if ((arcAngle > -360) && (arcAngle < 360))
				{
					pathData += "m " + x + "," + y;
					pathData += " l " + x1 + "," + y1;
				}
				else
				{
					pathData += "m " + x1 + "," + y1;
				}
				pathData += (arcAngle < 0) ? " at" : " wa";
				pathData += " " + left + "," + top + "," + right + "," + bottom + ", " + x1 + "," + y1 + ", " + x2 + "," + y2;
				pathData += ", " + left + "," + top + "," + right + "," + bottom + ", " + x2 + "," + y2 + ", " + x3 + "," + y3;
				pathData += " x";

				this._pathElement.v = pathData;

				return this;
			};

		});

		VectorElement.mixin(this, SVGWedge, VMLWedge);

	});

});

define('splunk/viz/MDataTarget',['require','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','jgatt','splunk/charting/Legend'],function(require)
{

	var jg_static = require("jg_global").jg_static;
	var jg_mixin = require("jg_global").jg_mixin;
	var MObservable = require("jgatt").events.MObservable;
	var MPropertyTarget = require("jgatt").properties.MPropertyTarget;
	var ObservableArrayProperty = require("jgatt").properties.ObservableArrayProperty;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var FunctionUtils = require("jgatt").utils.FunctionUtils;
	var MValidateTarget = require("jgatt").validation.MValidateTarget;
	var ValidatePass = require("jgatt").validation.ValidatePass;
	var Legend = require("splunk/charting/Legend");

	return jg_static(function(MDataTarget)
	{

		// Mixin

		this.mixin = function(base)
		{
			base = jg_mixin(this, MObservable, base);
			base = jg_mixin(this, MPropertyTarget, base);
			base = jg_mixin(this, MValidateTarget, base);
		};

		// Public Passes

		this.processDataPass = new ValidatePass("processData", 0.1);
		this.updateLegendLabelsPass = new ValidatePass("updateLegendLabels", 0.2);
		this.renderDataPass = new ValidatePass("renderData", 0.3);

		// Public Properties

		this.data = new ObservableProperty("data", Array, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.fields = new ObservableArrayProperty("fields", String, null)
			.onChanged(function(e)
			{
				this.invalidate("processDataPass");
			});

		this.legend = new ObservableProperty("legend", Legend, null)
			.onChanged(function(e)
			{
				if (e.target === this)
				{
					var oldLegend = e.oldValue;
					var newLegend = e.newValue;

					if (oldLegend)
					{
						oldLegend.removeEventListener("settingLabels", this._legend_settingLabels);
						oldLegend.unregister(this);
					}

					if (newLegend)
					{
						newLegend.register(this);
						newLegend.addEventListener("settingLabels", this._legend_settingLabels);
					}

					this.invalidate("updateLegendLabelsPass");
					return;
				}

				if (e.event === e.target.labelIndexMapChanged)
				{
					this.invalidate("renderDataPass");
					return;
				}
			});

		// Private Properties

		this._cachedData = null;
		this._cachedFields = null;
		this._cachedLegend = null;

		// Constructor

		this.constructor = function()
		{
			this._legend_settingLabels = FunctionUtils.bind(this._legend_settingLabels, this);
		};

		// Public Methods

		this.processData = function()
		{
			this.validatePreceding("processDataPass");

			if (this.isValid("processDataPass"))
				return;

			this.invalidate("updateLegendLabelsPass");

			var data = this._cachedData = this.getInternal("data") || [];
			var fields = this._cachedFields = this.getInternal("fields") || [];

			this.processDataOverride(data, fields);

			this.setValid("processDataPass");
		};

		this.updateLegendLabels = function()
		{
			this.validatePreceding("updateLegendLabelsPass");

			if (this.isValid("updateLegendLabelsPass"))
				return;

			this.invalidate("renderDataPass");

			var legend = this._cachedLegend = this.getInternal("legend");
			var labels = null;

			if (legend)
				labels = this.updateLegendLabelsOverride(this._cachedData, this._cachedFields);

			this.setValid("updateLegendLabelsPass");

			// this must run last to avoid recursion
			if (legend)
				legend.setLabels(this, labels);
		};

		this.renderData = function()
		{
			this.validatePreceding("renderDataPass");

			if (this.isValid("renderDataPass"))
				return;

			this.renderDataOverride(this._cachedData, this._cachedFields, this._cachedLegend);

			this.setValid("renderDataPass");
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			return null;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
		};

		// Private Methods

		this._legend_settingLabels = function(e)
		{
			this.validate("updateLegendLabelsPass");
		};

	});

});

define('splunk/mapping/layers/PieMarkerLayer',['require','leaflet','jg_global','jg_global','jgatt','jgatt','jgatt','jgatt','splunk/mapping/LatLon','splunk/mapping/LatLonBounds','splunk/mapping/layers/LayerBase','splunk/mapping/layers/VectorLayerBase','splunk/palettes/ColorPalette','splunk/palettes/ListColorPalette','splunk/vectors/Group','splunk/vectors/Wedge','splunk/viz/MDataTarget'],function(require)
{

	var Leaflet = require("leaflet");
	var jg_extend = require("jg_global").jg_extend;
	var jg_mixin = require("jg_global").jg_mixin;
	var Point = require("jgatt").geom.Point;
	var Rectangle = require("jgatt").geom.Rectangle;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var NumberUtils = require("jgatt").utils.NumberUtils;
	var LatLon = require("splunk/mapping/LatLon");
	var LatLonBounds = require("splunk/mapping/LatLonBounds");
	var LayerBase = require("splunk/mapping/layers/LayerBase");
	var VectorLayerBase = require("splunk/mapping/layers/VectorLayerBase");
	var ColorPalette = require("splunk/palettes/ColorPalette");
	var ListColorPalette = require("splunk/palettes/ListColorPalette");
	var Group = require("splunk/vectors/Group");
	var Wedge = require("splunk/vectors/Wedge");
	var MDataTarget = require("splunk/viz/MDataTarget");

	var PieMarkerLayer = jg_extend(VectorLayerBase, function(PieMarkerLayer, base)
	{

		base = jg_mixin(this, MDataTarget, base);

		// Public Properties

		this.markerColorPalette = new ObservableProperty("markerColorPalette", ColorPalette, new ListColorPalette([ 0x00CC00, 0xCCCC00, 0xCC0000 ], true))
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerOpacity = new ObservableProperty("markerOpacity", Number, 1)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value <= Infinity)) ? Math.min(value, 1) : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMinSize = new ObservableProperty("markerMinSize", Number, 10)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.markerMaxSize = new ObservableProperty("markerMaxSize", Number, 50)
			.writeFilter(function(value)
			{
				return ((value >= 0) && (value < Infinity)) ? value : 0;
			})
			.onChanged(function(e)
			{
				this.invalidate("renderDataPass");
			});

		this.wrapX = new ObservableProperty("wrapX", Boolean, true)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		this.wrapY = new ObservableProperty("wrapY", Boolean, false)
			.onChanged(function(e)
			{
				this.invalidate("renderPass");
			});

		// Private Properties

		this._seriesList = null;
		this._markerList = null;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this._seriesList = [];
			this._markerList = [];
		};

		// Public Methods

		this.getLatLonBounds = function(center)
		{
			if ((center != null) && !(center instanceof LatLon))
				throw new Error("Parameter center must be an instance of splunk.mapping.LatLon.");

			this.validate();

			var bounds = new LatLonBounds(Infinity, Infinity, -Infinity, -Infinity);

			var markerList = this._markerList;
			for (var i = 0, l = markerList.length; i < l; i++)
				bounds.expand(markerList[i].latLon.normalize(center));

			return bounds.isFinite() ? bounds : null;
		};

		// Protected Methods

		this.processDataOverride = function(data, fields)
		{
			var seriesList = this._seriesList;
			var numSeries = 0;
			var series;

			var markerList = this._markerList;
			var numMarkers = 0;
			var marker;

			var sliceList;
			var numSlices;
			var slice;

			var i;
			var j;

			var numRows = data.length;
			var numFields = fields.length;
			if ((numRows > 0) && (numFields > 2))
			{
				var vectorContainer = this.vectorContainer;

				var fieldLat = fields[0];
				var fieldLon = fields[1];
				var fieldSeries;

				var valueLat;
				var valueLon;
				var valueSeries;

				var magMin = Infinity;
				var magMax = -Infinity;
				var magSpan = 0;
				var mag;

				var sum;
				var angle1;
				var angle2;

				// create or reuse series
				for (i = 2; i < numFields; i++)
				{
					fieldSeries = fields[i];
					if (numSeries < seriesList.length)
					{
						series = seriesList[numSeries];
					}
					else
					{
						series = new Series();
						seriesList.push(series);
					}

					series.name = fieldSeries;

					numSeries++;
				}

				// create or reuse markers
				for (i = 0; i < numRows; i++)
				{
					obj = data[i];
					if (obj == null)
						continue;

					valueLat = NumberUtils.parseNumber(obj[fieldLat]);
					valueLon = NumberUtils.parseNumber(obj[fieldLon]);
					if (isNaN(valueLat) || isNaN(valueLon))
						continue;

					if (numMarkers < markerList.length)
					{
						marker = markerList[numMarkers];
					}
					else
					{
						marker = new PieMarker();
						marker.appendTo(vectorContainer);
						markerList.push(marker);
					}

					// create or reuse slices and compute marker magnitude
					sliceList = marker.sliceList;
					numSlices = 0;
					mag = 0;
					for (j = 0; j < numSeries; j++)
					{
						series = seriesList[j];

						valueSeries = NumberUtils.parseNumber(obj[series.name]);
						if (isNaN(valueSeries) || (valueSeries <= 0))
							continue;

						if (numSlices < sliceList.length)
						{
							slice = sliceList[numSlices];
						}
						else
						{
							slice = new PieSlice();
							slice.appendTo(marker);
							sliceList.push(slice);
						}

						slice.series = series;
						slice.value = valueSeries;

						mag += valueSeries;

						numSlices++;
					}

					if (numSlices === 0)
						continue;

					// record marker attributes
					marker.latLon = new LatLon(valueLat, valueLon);
					marker.data = obj;
					marker.fields = fields;
					marker.magnitude = mag;

					// update magnitude min and max
					if (mag < magMin)
						magMin = mag;
					if (mag > magMax)
						magMax = mag;

					// compute slice angles
					sum = 0;
					angle1 = 0;
					angle2 = 0;
					for (j = 0; j < numSlices; j++)
					{
						slice = sliceList[j];

						sum += slice.value;
						angle1 = angle2;
						angle2 = 360 * (sum / mag);

						slice.startAngle = angle1 - 90;
						slice.arcAngle = angle2 - angle1;
					}

					// dispose unused slices
					for (j = sliceList.length - 1; j >= numSlices; j--)
					{
						slice = sliceList.pop();
						slice.dispose();
					}

					numMarkers++;
				}

				// compute marker scales
				magSpan = magMax - magMin;
				for (i = 0; i < numMarkers; i++)
				{
					marker = markerList[i];
					marker.scale = (magSpan > 0) ? NumberUtils.minMax((marker.magnitude - magMin) / magSpan, 0, 1) : (1 / numMarkers);
				}
			}

			// dispose unused markers
			for (i = markerList.length - 1; i >= numMarkers; i--)
			{
				marker = markerList.pop();
				marker.dispose();
			}

			// dispose unused series
			for (i = seriesList.length - 1; i >= numSeries; i--)
				seriesList.pop();
		};

		this.updateLegendLabelsOverride = function(data, fields)
		{
			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var labels = (numSeries > 0) ? new Array(numSeries) : null;
			for (var i = 0; i < numSeries; i++)
				labels[i] = seriesList[i].name;
			return labels;
		};

		this.renderDataOverride = function(data, fields, legend)
		{
			this.invalidate("renderPass");

			var seriesList = this._seriesList;
			var numSeries = seriesList.length;
			var series;
			var seriesIndex;
			var seriesCount;

			var markerColorPalette = this.getInternal("markerColorPalette");
			var markerOpacity = this.getInternal("markerOpacity");
			var markerMinSize = this.getInternal("markerMinSize");
			var markerMaxSize = this.getInternal("markerMaxSize");
			var markerList = this._markerList;
			var numMarkers = markerList.length;
			var marker;

			var sliceList;
			var numSlices;
			var slice;

			var i;
			var j;

			// assign series colors
			seriesCount = legend ? legend.getNumLabels() : numSeries;
			for (i = 0; i < numSeries; i++)
			{
				series = seriesList[i];
				seriesIndex = legend ? legend.getLabelIndex(series.name) : i;
				series.color = markerColorPalette ? markerColorPalette.getColor(series.name, seriesIndex, seriesCount) : 0x000000;
			}

			// render pie slices
			for (i = 0; i < numMarkers; i++)
			{
				marker = markerList[i];
				sliceList = marker.sliceList;
				numSlices = sliceList.length;

				marker.radius = Math.round(NumberUtils.interpolate(markerMinSize, markerMaxSize, marker.scale)) / 2;
				marker.tooltipOffsetRadius = marker.radius;
				marker.display("none");  // fixes vml flicker

				for (j = 0; j < numSlices; j++)
				{
					slice = sliceList[j];
					slice.fillColor(slice.series.color);
					slice.fillOpacity(markerOpacity);
					slice.draw(0, 0, marker.radius, marker.radius, slice.startAngle, slice.arcAngle);
				}
			}
		};

		this.renderOverride = function(map)
		{
			base.renderOverride.call(this, map);

			var leafletMap = map.leafletMap;
			var centerLatLng = leafletMap.getCenter();

			var wrapX = this.getInternal("wrapX");
			var wrapY = this.getInternal("wrapY");

			var vectorBounds = this.vectorBounds;
			var minX = vectorBounds.minX;
			var minY = vectorBounds.minY;
			var maxX = vectorBounds.maxX;
			var maxY = vectorBounds.maxY;

			var markerList = this._markerList;
			var marker;
			var markerLatLng;
			var markerPoint;

			for (var i = 0, l = markerList.length; i < l; i++)
			{
				marker = markerList[i];
				markerLatLng = new Leaflet.LatLng(marker.latLon.lat, marker.latLon.lon);

				if (wrapX)
				{
					markerLatLng.lng -= centerLatLng.lng;
					markerLatLng.lng %= 360;
					if (markerLatLng.lng > 180)
						markerLatLng.lng -= 360;
					else if (markerLatLng.lng < -180)
						markerLatLng.lng += 360;
					markerLatLng.lng += centerLatLng.lng;
				}

				if (wrapY)
				{
					markerLatLng.lat -= centerLatLng.lat;
					markerLatLng.lat %= 180;
					if (markerLatLng.lat > 90)
						markerLatLng.lat -= 180;
					else if (markerLatLng.lat < -90)
						markerLatLng.lat += 180;
					markerLatLng.lat += centerLatLng.lat;
				}

				marker.tooltipLatLng = markerLatLng;

				markerPoint = leafletMap.latLngToLayerPoint(markerLatLng);

				marker.translate(markerPoint.x, markerPoint.y);
				if (((markerPoint.x + marker.radius) < minX) || ((markerPoint.x - marker.radius) > maxX) ||
				    ((markerPoint.y + marker.radius) < minY) || ((markerPoint.y - marker.radius) > maxY))
					marker.display("none");
				else
					marker.display(null);
			}
		};

	});

	var Series = jg_extend(Object, function(Series, base)
	{

		// Public Properties

		this.name = null;
		this.color = 0x000000;

		// Constructor

		this.constructor = function()
		{
		};

	});

	var PieMarker = jg_extend(Group, function(PieMarker, base)
	{

		// Public Properties

		this.sliceList = null;
		this.latLon = null;
		this.data = null;
		this.fields = null;
		this.magnitude = 0;
		this.scale = 0;
		this.radius = 0;
		this.tooltipLatLng = null;
		this.tooltipOffsetRadius = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);

			this.element[LayerBase.METADATA_KEY] = this;

			this.sliceList = [];
		};

		// Public Methods

		this.dispose = function()
		{
			var sliceList = this.sliceList;
			for (var i = sliceList.length - 1; i >= 0; i--)
				sliceList[i].dispose();

			this.element[LayerBase.METADATA_KEY] = null;

			base.dispose.call(this);
		};

	});

	var PieSlice = jg_extend(Wedge, function(PieSlice, base)
	{

		// Public Properties

		this.series = null;
		this.value = 0;
		this.startAngle = 0;
		this.arcAngle = 0;

		// Constructor

		this.constructor = function()
		{
			base.constructor.call(this);
		};

	});

	return PieMarkerLayer;

});

define('splunk/parsers/Parser',['require','jg_global'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;

	return jg_extend(Object, function(Parser, base)
	{

		// Public Methods

		this.stringToValue = function(str)
		{
			return null;
		};

		this.valueToString = function(value)
		{
			return null;
		};

	});

});

define('splunk/parsers/ParseUtils',['require','jg_global'],function(require)
{

	var jg_static = require("jg_global").jg_static;

	return jg_static(function(ParseUtils)
	{

		// Private Static Constants

		var _UNESCAPE_PATTERN = /\\([.\n\r]?)/g;
		var _ESCAPE_SLASH_PATTERN = /\\/g;
		var _ESCAPE_QUOTE_PATTERN = /"/g;

		// Public Static Methods

		ParseUtils.prepareArray = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "[")
				return null;

			if (str.charAt(length - 1) != "]")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var arr = [];
			var index = -1;
			var value;

			while (index < length)
			{
				index++;
				value = _readUntil(str, index, ",");
				index += value.length;

				value = ParseUtils.trimWhiteSpace(value);
				if (value || (index < length) || (arr.length > 0))
					arr.push(ParseUtils.unescapeString(value));
			}

			return arr;
		};

		ParseUtils.prepareObject = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "{")
				return null;

			if (str.charAt(length - 1) != "}")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var obj = {};
			var index = 0;
			var key;
			var value;

			while (index < length)
			{
				key = _readUntil(str, index, ":");
				index += key.length + 1;

				if (index > length)
					break;

				value = _readUntil(str, index, ",");
				index += value.length + 1;

				key = ParseUtils.unescapeString(key);
				if (key)
					obj[key] = ParseUtils.unescapeString(value);
			}

			return obj;
		};

		ParseUtils.prepareTuple = function(str)
		{
			if (!str || (typeof str !== "string"))
				return null;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return null;

			var length = str.length;
			if (length < 2)
				return null;

			if (str.charAt(0) != "(")
				return null;

			if (str.charAt(length - 1) != ")")
				return null;

			str = str.substring(1, length - 1);
			length = str.length;

			var arr = [];
			var index = -1;
			var value;

			while (index < length)
			{
				index++;
				value = _readUntil(str, index, ",");
				index += value.length;

				value = ParseUtils.trimWhiteSpace(value);
				if (value || (index < length) || (arr.length > 0))
					arr.push(ParseUtils.unescapeString(value));
			}

			return arr;
		};

		ParseUtils.unescapeString = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			if (!str)
				return str;

			str = ParseUtils.trimWhiteSpace(str);
			if (!str)
				return str;

			var length = str.length;
			if (length < 2)
				return str;

			if (str.charAt(0) != "\"")
				return str;

			if (str.charAt(length - 1) != "\"")
				return str;

			str = str.substring(1, length - 1);
			if (!str)
				return str;

			str = str.replace(_UNESCAPE_PATTERN, "$1");

			return str;
		};

		ParseUtils.escapeString = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			// two simple replace calls are faster than str.replace(/([\\"])/g, "\\$1")
			str = str.replace(_ESCAPE_SLASH_PATTERN, "\\\\");
			str = str.replace(_ESCAPE_QUOTE_PATTERN, "\\\"");

			return "\"" + str + "\"";
		};

		ParseUtils.trimWhiteSpace = function(str)
		{
			if ((str == null) || (typeof str !== "string"))
				return null;

			if (!str)
				return str;

			var startIndex = 0;
			var endIndex = str.length - 1;

			for (startIndex; startIndex <= endIndex; startIndex++)
			{
				if (!ParseUtils.isWhiteSpace(str.charAt(startIndex)))
					break;
			}

			for (endIndex; endIndex >= startIndex; endIndex--)
			{
				if (!ParseUtils.isWhiteSpace(str.charAt(endIndex)))
					break;
			}

			return str.substring(startIndex, endIndex + 1);
		};

		ParseUtils.isWhiteSpace = function(ch)
		{
			return ((ch === " ") || (ch === "\t") || (ch === "\n") || (ch === "\r"));
		};

		// Private Static Methods

		var _readUntil = function(str, startIndex, endChar)
		{
			var substr = "";

			var index = startIndex;
			var length = str.length;
			var ch;
			var isQuote = false;
			var nestLevel = 0;
			var nestBeginChar;
			var nestEndChar;

			while (index < length)
			{
				ch = str.charAt(index);
				if (isQuote)
				{
					if (ch == "\"")
					{
						isQuote = false;
					}
					else if (ch == "\\")
					{
						substr += ch;
						index++;
						ch = str.charAt(index);
					}
				}
				else if (nestLevel > 0)
				{
					if (ch == nestEndChar)
						nestLevel--;
					else if (ch == nestBeginChar)
						nestLevel++;
					else if (ch == "\"")
						isQuote = true;
				}
				else if (ch != endChar)
				{
					if (ch == "[")
					{
						nestLevel = 1;
						nestBeginChar = "[";
						nestEndChar = "]";
					}
					else if (ch == "{")
					{
						nestLevel = 1;
						nestBeginChar = "{";
						nestEndChar = "}";
					}
					else if (ch == "(")
					{
						nestLevel = 1;
						nestBeginChar = "(";
						nestEndChar = ")";
					}
					else if (ch == "\"")
					{
						isQuote = true;
					}
				}
				else
				{
					break;
				}

				substr += ch;
				index++;
			}

			return substr;
		};

	});

});

define('splunk/parsers/NumberParser',['require','jg_global','splunk/parsers/Parser','splunk/parsers/ParseUtils'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");

	return jg_extend(Parser, function(NumberParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		NumberParser.getInstance = function()
		{
			if (!_instance)
				_instance = new NumberParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			str = ParseUtils.trimWhiteSpace(str);
			return str ? Number(str) : NaN;
		};

		this.valueToString = function(value)
		{
			return (typeof value === "number") ? String(value) : String(NaN);
		};

	});

});

define('splunk/mapping/parsers/LatLonBoundsParser',['require','jg_global','splunk/mapping/LatLonBounds','splunk/parsers/NumberParser','splunk/parsers/Parser','splunk/parsers/ParseUtils'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var LatLonBounds = require("splunk/mapping/LatLonBounds");
	var NumberParser = require("splunk/parsers/NumberParser");
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");

	return jg_extend(Parser, function(LatLonBoundsParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		LatLonBoundsParser.getInstance = function()
		{
			if (!_instance)
				_instance = new LatLonBoundsParser();
			return _instance;
		};

		// Protected Properties

		this.numberParser = null;

		// Constructor

		this.constructor = function()
		{
			this.numberParser = NumberParser.getInstance();
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var values = ParseUtils.prepareTuple(str);
			if (!values)
				return null;

			var latLonBounds = new LatLonBounds();

			var numValues = values.length;
			if (numValues > 0)
				latLonBounds.s = this.numberParser.stringToValue(values[0]);
			if (numValues > 1)
				latLonBounds.w = this.numberParser.stringToValue(values[1]);
			if (numValues > 2)
				latLonBounds.n = this.numberParser.stringToValue(values[2]);
			if (numValues > 3)
				latLonBounds.e = this.numberParser.stringToValue(values[3]);

			return latLonBounds;
		};

		this.valueToString = function(value)
		{
			var latLonBounds = (value instanceof LatLonBounds) ? value : null;
			if (!latLonBounds)
				return null;

			var str = "";

			str += this.numberParser.valueToString(latLonBounds.s) + ",";
			str += this.numberParser.valueToString(latLonBounds.w) + ",";
			str += this.numberParser.valueToString(latLonBounds.n) + ",";
			str += this.numberParser.valueToString(latLonBounds.e);

			return "(" + str + ")";
		};

	});

});

define('splunk/mapping/parsers/LatLonParser',['require','jg_global','splunk/mapping/LatLon','splunk/parsers/NumberParser','splunk/parsers/Parser','splunk/parsers/ParseUtils'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var LatLon = require("splunk/mapping/LatLon");
	var NumberParser = require("splunk/parsers/NumberParser");
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");

	return jg_extend(Parser, function(LatLonParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		LatLonParser.getInstance = function()
		{
			if (!_instance)
				_instance = new LatLonParser();
			return _instance;
		};

		// Protected Properties

		this.numberParser = null;

		// Constructor

		this.constructor = function()
		{
			this.numberParser = NumberParser.getInstance();
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var values = ParseUtils.prepareTuple(str);
			if (!values)
				return null;

			var latLon = new LatLon();

			var numValues = values.length;
			if (numValues > 0)
				latLon.lat = this.numberParser.stringToValue(values[0]);
			if (numValues > 1)
				latLon.lon = this.numberParser.stringToValue(values[1]);

			return latLon;
		};

		this.valueToString = function(value)
		{
			var latLon = (value instanceof LatLon) ? value : null;
			if (!latLon)
				return null;

			var str = "";

			str += this.numberParser.valueToString(latLon.lat) + ",";
			str += this.numberParser.valueToString(latLon.lon);

			return "(" + str + ")";
		};

	});

});

define('splunk/palettes/FieldColorPalette',['require','jg_global','jgatt','splunk/palettes/ColorPalette'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var ObservableProperty = require("jgatt").properties.ObservableProperty;
	var ColorPalette = require("splunk/palettes/ColorPalette");

	return jg_extend(ColorPalette, function(FieldColorPalette, base)
	{

		// Private Static Methods

		var _cloneFieldColors = function(fieldColors)
		{
			var fieldColors2 = {};
			for (var field in fieldColors)
			{
				if (fieldColors[field] != null)
					fieldColors2[field] = Number(fieldColors[field]);
			}
			return fieldColors2;
		};

		// Public Properties

		this.fieldColors = new ObservableProperty("fieldColors", Object, {})
			.readFilter(function(value)
			{
				return _cloneFieldColors(value);
			})
			.writeFilter(function(value)
			{
				return _cloneFieldColors(value);
			});

		this.defaultColorPalette = new ObservableProperty("defaultColorPalette", ColorPalette, null);

		// Constructor

		this.constructor = function(fieldColors, defaultColorPalette)
		{
			base.constructor.call(this);

			if (fieldColors != null)
				this.set(this.fieldColors, fieldColors);
			if (defaultColorPalette != null)
				this.set(this.defaultColorPalette, defaultColorPalette);
		};

		// Protected Methods

		this.getColorOverride = function(properties, field, index, count)
		{
			if (field)
			{
				var color = properties.fieldColors[field];
				if ((color != null) && !isNaN(color))
					return color;
			}

			if (properties.defaultColorPalette)
				return properties.defaultColorPalette.getColor(field, index, count);

			return 0x000000;
		};

	});

});

define('splunk/parsers/StringParser',['require','jg_global','splunk/parsers/Parser'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Parser = require("splunk/parsers/Parser");

	return jg_extend(Parser, function(StringParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		StringParser.getInstance = function()
		{
			if (!_instance)
				_instance = new StringParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			return ((str == null) || (typeof str !== "string")) ? null : str;
		};

		this.valueToString = function(value)
		{
			return (value == null) ? null : String(value);
		};

	});

});

define('splunk/parsers/ArrayParser',['require','jg_global','jgatt','splunk/parsers/Parser','splunk/parsers/ParseUtils','splunk/parsers/StringParser'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Dictionary = require("jgatt").utils.Dictionary;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");
	var StringParser = require("splunk/parsers/StringParser");

	return jg_extend(Parser, function(ArrayParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ArrayParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ArrayParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var array = ParseUtils.prepareArray(str);
			if (!array)
				return null;

			var elementParser = this.elementParser;
			for (var i = 0, l = array.length; i < l; i++)
				array[i] = elementParser.stringToValue(array[i]);

			return array;
		};

		this.valueToString = function(value)
		{
			var array = (value instanceof Array) ? value : null;
			if (!array)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			var elementValue;
			for (var i = 0, l = array.length; i < l; i++)
			{
				elementValue = array[i];
				if (str)
					str += ",";
				if (elementParser instanceof StringParser)
					str += ParseUtils.escapeString(elementParser.valueToString(elementValue));
				else
					str += elementParser.valueToString(elementValue);
			}

			return "[" + str + "]";
		};

	});

});

define('splunk/parsers/BooleanParser',['require','jg_global','splunk/parsers/Parser','splunk/parsers/ParseUtils'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");

	return jg_extend(Parser, function(BooleanParser, base)
	{

		// Private Static Properties

		var _instance = null;

		// Public Static Methods

		BooleanParser.getInstance = function()
		{
			if (!_instance)
				_instance = new BooleanParser();
			return _instance;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			str = ParseUtils.trimWhiteSpace(str);
			if (str)
				str = str.toLowerCase();
			return ((str === "true") || (str === "t") || (str === "1"));
		};

		this.valueToString = function(value)
		{
			return value ? "true" : "false";
		};

	});

});

define('splunk/parsers/ObjectParser',['require','jg_global','jgatt','splunk/parsers/Parser','splunk/parsers/ParseUtils','splunk/parsers/StringParser'],function(require)
{

	var jg_extend = require("jg_global").jg_extend;
	var Dictionary = require("jgatt").utils.Dictionary;
	var Parser = require("splunk/parsers/Parser");
	var ParseUtils = require("splunk/parsers/ParseUtils");
	var StringParser = require("splunk/parsers/StringParser");

	return jg_extend(Parser, function(ObjectParser, base)
	{

		// Private Static Properties

		var _instances = new Dictionary();

		// Public Static Methods

		ObjectParser.getInstance = function(elementParser)
		{
			var instance = _instances.get(elementParser);
			if (!instance)
				instance = _instances.set(elementParser, new ObjectParser(elementParser));
			return instance;
		};

		// Protected Properties

		this.elementParser = null;

		// Constructor

		this.constructor = function(elementParser)
		{
			if (elementParser == null)
				throw new Error("Parameter elementParser must be non-null.");
			if (!(elementParser instanceof Parser))
				throw new Error("Parameter elementParser must be an instance of splunk.parsers.Parser.");

			this.elementParser = elementParser;
		};

		// Public Methods

		this.stringToValue = function(str)
		{
			var map = ParseUtils.prepareObject(str);
			if (!map)
				return null;

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
					map[key] = elementParser.stringToValue(map[key]);
			}

			return map;
		};

		this.valueToString = function(value)
		{
			var map = (value instanceof Object) ? value : null;
			if (!map)
				return null;

			var str = "";

			var elementParser = this.elementParser;
			for (var key in map)
			{
				if (map.hasOwnProperty(key))
				{
					if (str)
						str += ",";
					str += ParseUtils.escapeString(key) + ":";
					if (elementParser instanceof StringParser)
						str += ParseUtils.escapeString(elementParser.valueToString(map[key]));
					else
						str += elementParser.valueToString(map[key]);
				}
			}

			return "{" + str + "}";
		};

	});

});

define('views/shared/Map',['require','exports','module','jquery','splunk.i18n','splunk.util','jgatt','jgatt','splunk/charting/ExternalLegend','splunk/mapping/Map','splunk/mapping/layers/PieMarkerLayer','splunk/mapping/parsers/LatLonBoundsParser','splunk/mapping/parsers/LatLonParser','splunk/palettes/FieldColorPalette','splunk/palettes/ListColorPalette','splunk/parsers/ArrayParser','splunk/parsers/BooleanParser','splunk/parsers/NumberParser','splunk/parsers/ObjectParser','splunk/parsers/StringParser','views/Base','util/console'],function(require, exports, module) {

	var $ = require('jquery');
	var SplunkI18N = require('splunk.i18n');
	var _ = SplunkI18N._;
	var SplunkUtil = require('splunk.util');
	var sprintf = SplunkUtil.sprintf;
	var FunctionUtils = require('jgatt').utils.FunctionUtils;
	var ValidateQueue = require('jgatt').validation.ValidateQueue;
	var ExternalLegend = require('splunk/charting/ExternalLegend');
	var Map = require('splunk/mapping/Map');
	var PieMarkerLayer = require('splunk/mapping/layers/PieMarkerLayer');
	var LatLonBoundsParser = require('splunk/mapping/parsers/LatLonBoundsParser');
	var LatLonParser = require('splunk/mapping/parsers/LatLonParser');
	var FieldColorPalette = require('splunk/palettes/FieldColorPalette');
	var ListColorPalette = require('splunk/palettes/ListColorPalette');
	var ArrayParser = require('splunk/parsers/ArrayParser');
	var BooleanParser = require('splunk/parsers/BooleanParser');
	var NumberParser = require('splunk/parsers/NumberParser');
	var ObjectParser = require('splunk/parsers/ObjectParser');
	var StringParser = require('splunk/parsers/StringParser');
	var Base = require('views/Base');
    var console = require('util/console');

	var _DEFAULT_PROPERTY_VALUES = {
		"fieldColors": "",
		"seriesColors": "[" +
			"0x6CB8CA,0xFAC61D,0xD85E3D,0x956E96,0xF7912C,0x9AC23C,0x5479AF,0x999755,0xDD87B0,0x65AA82," +
			"0xA7D4DF,0xFCDD77,0xE89E8B,0xBFA8C0,0xFABD80,0xC2DA8A,0x98AFCF,0xC2C199,0xEBB7D0,0xA3CCB4," +
			"0x416E79,0x967711,0x823825,0x59425A,0x94571A,0x5C7424,0x324969,0x5C5B33,0x85516A,0x3D664E" +
		"]",
		"data.maxClusters": "100",
		"tileLayer.url": "/splunkd/__raw/services/mbtiles/splunk-tiles/{z}/{x}/{y}",
		"tileLayer.subdomains": "[a,b,c]",
		"tileLayer.minZoom": "0",
		"tileLayer.maxZoom": "7",
		"tileLayer.invertY": "false",
		"tileLayer.attribution": "",
		"markerLayer.markerOpacity": "1",
		"markerLayer.markerMinSize": "10",
		"markerLayer.markerMaxSize": "50"
	};

	var _R_PROPERTY_PREFIX = /^display\.visualizations\.mapping\./;

	return Base.extend({

		// Public Properties

		moduleId: module.id,

		// Private Properties

		_map: null,
		_markerLayer: null,
		_externalLegend: null,
		_fieldColorPalette: null,
		_seriesColorPalette: null,
		_propertyValues: null,
		_booleanParser: null,
		_numberParser: null,
		_stringParser: null,
		_numberArrayParser: null,
		_stringArrayParser: null,
		_numberObjectParser: null,
		_latLonParser: null,
		_latLonBoundsParser: null,
		_maxClusters: 100,
		_isPrinting: false,
		_prePrintCenter: null,
		_prePrintZoom: 0,

		// Constructor

		initialize: function(options) {
			Base.prototype.initialize.apply(this, arguments);

			this.options = options || {};
			this.$el.width(this.options.width || "100%");
			this.$el.height(this.options.height || "400px");

			if (this.model.searchResultsRow)
				this.model.searchResultsRow.on("change destroy error", this.updateData, this);
			else if (this.model.searchResultsColumn)
				this.model.searchResultsColumn.on("change destroy error", this.updateData, this);

			if (this.model.state)
				this.model.state.on("change", this._state_change, this);

			this._map_boundsChanged = FunctionUtils.bind(this._map_boundsChanged, this);
			this._map_mapClicked = FunctionUtils.bind(this._map_mapClicked, this);
			this._document_PrintStart = FunctionUtils.bind(this._document_PrintStart, this);
			this._document_PrintEnd = FunctionUtils.bind(this._document_PrintEnd, this);

			this._propertyValues = {};

			this._booleanParser = BooleanParser.getInstance();
			this._numberParser = NumberParser.getInstance();
			this._stringParser = StringParser.getInstance();
			this._numberArrayParser = ArrayParser.getInstance(this._numberParser);
			this._stringArrayParser = ArrayParser.getInstance(this._stringParser);
			this._numberObjectParser = ObjectParser.getInstance(this._numberParser);
			this._latLonParser = LatLonParser.getInstance();
			this._latLonBoundsParser = LatLonBoundsParser.getInstance();

			this._seriesColorPalette = new ListColorPalette();
			this._fieldColorPalette = new FieldColorPalette(null, this._seriesColorPalette);

			this._externalLegend = new ExternalLegend();
			this._externalLegend.connect();

			this._markerLayer = new PieMarkerLayer();
			this._markerLayer.set("legend", this._externalLegend);
			this._markerLayer.set("markerColorPalette", this._fieldColorPalette);

			this._map = new Map();
			this._map.formatNumber = this._formatNumber;
			this._map.formatDegrees = this._formatDegrees;
			this._map.addLayer(this._markerLayer);
			this._map.addEventListener("boundsChanged", this._map_boundsChanged);
			this._map.addEventListener("mapClicked", this._map_mapClicked);
			this._map.appendTo(this.$el);
			this._map.fitWorld(true);

			$(document).bind("PrintStart", this._document_PrintStart);
			$(document).bind("PrintEnd", this._document_PrintEnd);

			this.updateData();
			this.updateProperties();
			this._updateDataBounds();
		},

		// Public Methods

		getLatLonBounds: function() {
			return this._map.getLatLonBounds().normalize();
		},

		getPostProcessSearch: function() {
			var bounds = this._map.getLatLonBounds().normalize();
			return "geofilter south=" + bounds.s + " west=" + bounds.w + " north=" + bounds.n + " east=" + bounds.e + " maxclusters=" + this._maxClusters;
		},

		getMaxClusters: function() {
			return this._maxClusters;
		},

		updateData: function() {
			var extractedData = null;
			if (this.model.searchResultsRow)
				extractedData = this._extractRowData(this.model.searchResultsRow);
			else if (this.model.searchResultsColumn)
				extractedData = this._extractColumnData(this.model.searchResultsColumn);

            console.debug('Updating map data to', extractedData);
			this._markerLayer.set("data", extractedData ? extractedData.data : null);
			this._markerLayer.set("fields", extractedData ? this._filterFields(extractedData.fields) : null);
		},

		updateProperties: function() {
			var curValues = this._propertyValues;
			var newValues = {};
			var p;

			// set null values for all existing properties
			// if they are not overridden by either the default or state properties, they will be cleared
			for (p in curValues) {
				if (curValues.hasOwnProperty(p))
					newValues[p] = null;
			}

			// copy default property values
			var defaultValues = _DEFAULT_PROPERTY_VALUES;
			for (p in defaultValues) {
				if (defaultValues.hasOwnProperty(p))
					newValues[p] = defaultValues[p];
			}

			// copy state property values
			var stateValues = this.model.state ? this.model.state.toJSON() : {};
			var rPrefix = _R_PROPERTY_PREFIX;
			for (p in stateValues) {
				if (stateValues.hasOwnProperty(p) && rPrefix.test(p))
					newValues[p.replace(rPrefix, "")] = stateValues[p];
			}

			// apply map viewport properties in order
			// zoom must be first for Leaflet to do the right thing
			if (newValues.hasOwnProperty("map.zoom"))
				this._setMapProperty("map.zoom", newValues["map.zoom"]);
			if (newValues.hasOwnProperty("map.center"))
				this._setMapProperty("map.center", newValues["map.center"]);
			if (newValues.hasOwnProperty("map.fitBounds"))
				this._setMapProperty("map.fitBounds", newValues["map.fitBounds"]);

			// apply remaining properties
			// the viewport properties haven't changed, so they will be ignored by _setMapProperty
			for (p in newValues) {
				if (newValues.hasOwnProperty(p))
					this._setMapProperty(p, newValues[p]);
			}
		},

		render: function() {
			return this;
		},

		remove: function() {
			$(document).unbind("PrintStart", this._document_PrintStart);
			$(document).unbind("PrintEnd", this._document_PrintEnd);

			this._map.removeEventListener("boundsChanged", this._map_boundsChanged);
			this._map.removeEventListener("mapClicked", this._map_mapClicked);
			this._map.dispose();

			this._externalLegend.close();

			if (this.model.searchResultsRow)
				this.model.searchResultsRow.off("change destroy error", this.updateData, this);
			else if (this.model.searchResultsColumn)
				this.model.searchResultsColumn.off("change destroy error", this.updateData, this);

			if (this.model.state)
				this.model.state.off("change", this._state_change, this);

			return Base.prototype.remove.apply(this, arguments);
		},

		onShow: function() {
			this._map.updateSize();
			this._map.validate();
			Base.prototype.onShow.call(this);
		},

		// Private Methods

		_updateDataBounds: function() {
			var model = this.model.state;
			if (!model)
				return;

			var bounds = this._map.getLatLonBounds().normalize();
			model.set({ "display.visualizations.mapping.data.bounds": this._latLonBoundsParser.valueToString(bounds) });
		},

		_needsPropertyUpdate: function(changedProperties) {
			if (!changedProperties)
				return false;

			for (var p in changedProperties) {
				if (changedProperties.hasOwnProperty(p) && (p !== "display.visualizations.mapping.data.bounds"))
					return true;
			}

			return false;
		},

		_extractRowData: function(model) {
			var extractedData = {};

			var fields = model.get("fields");
			var rows = model.get("rows");
			if (fields && rows) {
				var numFields = fields.length;
				var numRows = rows.length;
				var numEntries;
				var row;
				var obj;
				var i;
				var j;

				extractedData.fields = fields.concat();
				extractedData.data = [];
				for (i = 0; i < numRows; i++) {
					row = rows[i];
					numEntries = Math.min(row.length, numFields);
					obj = {};
					for (j = 0; j < numEntries; j++)
						obj[fields[j]] = row[j];
					extractedData.data.push(obj);
				}
			}

			return extractedData;
		},

		_extractColumnData: function(model) {
			var extractedData = {};

			var fields = model.get("fields");
			var columns = model.get("columns");
			if (fields && columns) {
				var numColumns = Math.min(fields.length, columns.length);
				var numRows = (numColumns > 0) ? columns[0].length : 0;
				var obj;
				var i;
				var j;

				for (i = 1; i < numColumns; i++)
					numRows = Math.min(numRows, columns[i].length);

				extractedData.fields = fields.slice(0, numColumns);
				extractedData.data = [];
				for (i = 0; i < numRows; i++) {
					obj = {};
					for (j = 0; j < numColumns; j++)
						obj[fields[j]] = columns[j][i];
					extractedData.data.push(obj);
				}
			}

			return extractedData;
		},

		_filterFields: function(fields) {
			if (!fields)
				return null;

			var filteredFields = [];
			var field;
			for (var i = 0, l = fields.length; i < l; i++) {
				field = fields[i];
				if (field && (field.charAt(0) !== "_"))
					filteredFields.push(field);
			}
			return filteredFields;
		},

		_setMapProperty: function(propertyName, propertyValue) {
			propertyValue = (propertyValue != null) ? String(propertyValue) : null;
			if (this._propertyValues[propertyName] == propertyValue)
				return;

			if (propertyValue != null)
				this._propertyValues[propertyName] = propertyValue;
			else
				delete this._propertyValues[propertyName];

			switch (propertyName) {
				// global properties
				case "fieldColors":
					this._fieldColorPalette.set("fieldColors", this._numberObjectParser.stringToValue(propertyValue));
					break;
				case "seriesColors":
					this._seriesColorPalette.set("colors", this._numberArrayParser.stringToValue(propertyValue));
					break;

				// data properties
				case "data.maxClusters":
					var maxClusters = this._numberParser.stringToValue(propertyValue);
					this._maxClusters = (maxClusters < Infinity) ? Math.max(Math.floor(maxClusters), 1) : 100;
					break;

				// map properties
				case "map.center":
					var center = this._latLonParser.stringToValue(propertyValue);
					if (center)
						this._map.set("center", center);
					break;
				case "map.zoom":
					var zoom = this._numberParser.stringToValue(propertyValue);
					if (!isNaN(zoom))
						this._map.set("zoom", zoom);
					break;
				case "map.fitBounds":
					var fitBounds = this._latLonBoundsParser.stringToValue(propertyValue);
					if (fitBounds)
						this._map.fitBounds(fitBounds);
					break;

				// tileLayer properties
				case "tileLayer.url":
					this._map.set("tileURL", this._resolveURL(propertyValue));
					break;
				case "tileLayer.subdomains":
					this._map.set("tileSubdomains", this._stringArrayParser.stringToValue(propertyValue));
					break;
				case "tileLayer.minZoom":
					this._map.set("tileMinZoom", this._numberParser.stringToValue(propertyValue));
					break;
				case "tileLayer.maxZoom":
					this._map.set("tileMaxZoom", this._numberParser.stringToValue(propertyValue));
					break;
				case "tileLayer.invertY":
					this._map.set("tileInvertY", this._booleanParser.stringToValue(propertyValue));
					break;
				case "tileLayer.attribution":
					this._map.set("tileAttribution", this._stringParser.stringToValue(propertyValue));
					break;

				// markerLayer properties
				case "markerLayer.markerOpacity":
					this._markerLayer.set("markerOpacity", this._numberParser.stringToValue(propertyValue));
					break;
				case "markerLayer.markerMinSize":
					this._markerLayer.set("markerMinSize", this._numberParser.stringToValue(propertyValue));
					break;
				case "markerLayer.markerMaxSize":
					this._markerLayer.set("markerMaxSize", this._numberParser.stringToValue(propertyValue));
					break;
			}
		},

		_resolveURL: function(propertyValue) {
			var propertyValue2 = propertyValue ? SplunkUtil.trim(propertyValue) : propertyValue;
			if (propertyValue2 && (propertyValue2.charAt(0) === "/")) {
				var hadTrailingSlash = (propertyValue2.charAt(propertyValue2.length - 1) === "/");
				propertyValue2 = SplunkUtil.make_url(propertyValue2);
				var hasTrailingSlash = (propertyValue2.charAt(propertyValue2.length - 1) === "/");
				if (hasTrailingSlash != hadTrailingSlash)
					propertyValue2 = hadTrailingSlash ? propertyValue2 + "/" : propertyValue2.substring(0, propertyValue2.length - 1);
				propertyValue = propertyValue2;
			}
			return propertyValue;
		},

		_formatNumber: function(num) {
			var pos = Math.abs(num);
			if ((pos > 0) && ((pos < 1e-3) || (pos >= 1e9)))
				return SplunkI18N.format_scientific(num, "##0E0");
			return SplunkI18N.format_decimal(num);
		},

		_formatDegrees: function(degrees, orientation) {
			var deg = Math.abs(degrees);
			var degInt = Math.floor(deg);
			var degStr = ("" + degInt);
			var min = (deg - degInt) * 60;
			var minInt = Math.floor(min);
			var minStr = (minInt < 10) ? ("0" + minInt) : ("" + minInt);
			var sec = (min - minInt) * 60;
			var secInt = Math.floor(sec);
			var secStr = (secInt < 10) ? ("0" + secInt) : ("" + secInt);

			var dirStr = "";
			if (degrees > 0)
				dirStr = (orientation === "ns") ? _("N") : _("E");
			else if (degrees < 0)
				dirStr = (orientation === "ns") ? _("S") : _("W");

			if (secInt > 0)
				return sprintf("%(degrees)s\u00B0%(minutes)s'%(seconds)s\"%(direction)s", { degrees: degStr, minutes: minStr, seconds: secStr, direction: dirStr });
			if (minInt > 0)
				return sprintf("%(degrees)s\u00B0%(minutes)s'%(direction)s", { degrees: degStr, minutes: minStr, direction: dirStr });
			return sprintf("%(degrees)s\u00B0%(direction)s", { degrees: degStr, direction: dirStr });
		},

		_state_change: function() {
			if (this.model.state.get("display.visualizations.type") === "mapping") {
				var changedProperties = this.model.state.filterChangedByWildcards(_R_PROPERTY_PREFIX, { allowEmpty: true });
				if (this._needsPropertyUpdate(changedProperties))
					this.updateProperties();
			}
		},

		_map_boundsChanged: function(e) {
			if (this._isPrinting)
				return;

			this._updateDataBounds();
			this.trigger("boundsChanged", {});
		},

		_map_mapClicked: function(e) {
			if (this._isPrinting)
				return;

			this.trigger("mapClicked", { data: e.data, fields: e.fields, altKey: e.altKey, ctrlKey: e.ctrlKey, shiftKey: e.shiftKey });
		},

		_document_PrintStart: function(e) {
			if (this._isPrinting)
				return;

			this._isPrinting = true;

			this._prePrintCenter = this._map.get("center");
			this._prePrintZoom = this._map.get("zoom");

			this._map.updateSize();

			var dataBounds = this._markerLayer.getLatLonBounds(this._prePrintCenter);
			if (dataBounds)
				this._map.fitBounds(dataBounds);

			ValidateQueue.validateAll();
		},

		_document_PrintEnd: function(e) {
			if (!this._isPrinting)
				return;

			this._map.set("center", this._prePrintCenter);
			this._map.set("zoom", this._prePrintZoom);

			this._prePrintCenter = null;
			this._prePrintZoom = 0;
			this._isPrinting = false;
		}

	}, {
		DEFAULT_PROPERTY_VALUES: _DEFAULT_PROPERTY_VALUES
	});

});

define('splunkjs/mvc/splunkmapview',['require','exports','module','jquery','underscore','./mvc','models/Base','./basesplunkview','./messages','splunk.util','views/shared/Map','backbone','./utils','uri/route','JS_CACHE/config','util/console','jquery.ui.resizable','splunk.util','./tokenawaremodel'],function(require, exports, module) {
    var $ = require("jquery");
    var _ = require("underscore");
    var mvc = require("./mvc");
    var BaseModel = require("models/Base");
    var BaseSplunkView = require("./basesplunkview");
    var Messages = require("./messages");
    var SplunkUtil = require("splunk.util");
    var Map = require("views/shared/Map");
    var Backbone = require('backbone');
    var utils = require('./utils');
    var route = require("uri/route");
    var config = require('JS_CACHE/config');
    var console = require('util/console');
    var resizable = require('jquery.ui.resizable');
    var util = require('splunk.util');
    var TokenAwareModel = require('./tokenawaremodel');

    var TileSources = {
        openStreetMap: {
            url: 'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
            maxZoom: 18
        }
        // TODO: dynamically add Splunk tile source info
    };

    var SplunkMapView = BaseSplunkView.extend({
        moduleId: module.id,

        className: "splunk-map",

        options: {
            data: "preview",
            height: '400px',
            tileSource: undefined,
            tileUrl: undefined,
            maxZoom: undefined,
            drilldown: true,
            drilldownRedirect: true,
            resizable: false
        },

        omitFromSettings: ['el', 'reportModel'],

        initialize: function() {
            this.configure();
            this.model = this.options.reportModel || TokenAwareModel._createReportModel();

            this.settings._sync = utils.syncModels(this.settings, this.model, {
                auto: true,
                prefix: 'display.visualizations.',
                alias: {
                    tileUrl: 'display.visualizations.mapping.tileLayer.url',
                    maxZoom: 'display.visualizations.mapping.tileLayer.maxZoom'
                },
                exclude: ['managerid','data','tileSource','drilldown','type', 'drilldownRedirect']
            });

            this.results = new Backbone.Model({
                columns: [],
                fields: []
            });

            this._currentHeight = parseInt(this.settings.get('height'), 10);

            // initialize containers as detached DOM
            this.$map = $('<div></div>');
            this.$msg = $('<div></div>');

            this.createMap = _.debounce(_.bind(this.createMap, this), 0);
            this.createMap();
            this.settings.on('change:tileSource', this._updateTileSource, this);
            this._updateTileSource();
            this.bindToComponentSetting('managerid', this._onManagerChange, this);
        },

        _updateTileSource: function() {
            if(this.settings.has('tileSource')) {
                var tileSource = TileSources[this.settings.get('tileSource')];
                if(tileSource) {
                    this.settings.set({
                        tileUrl: tileSource.url,
                        maxZoom: tileSource.maxZoom
                    });
                } else {
                    console.warn('Invalid tileSource parameter value=%o', tileSource);
                }
            }
        },

        destroyMap: function() {
            if(this.map) {
                this.map.off();
                this.map.remove();
                this.map = null;
            }
        },

        createMap: function() {
            this.destroyMap();
            this.map = new Map({
                el: this.$map.appendTo(this.el),
                model: {
                    searchResultsColumn: this.results,
                    state: this.model
                },
                height: this.settings.get('height')
            });
            this.map.on('boundsChanged', this._onBoundsChanged, this);
            this.map.on('mapClicked', this._onMapClicked, this);

            // force data to update according to new map bounds
            this._onBoundsChanged();

            // Setup resizing
            this._onResizeMouseup = _.bind(this._onResizeMouseup, this);
            this.settings.on('change:resizable', function(model, value, options) {
                value ? this._enableResize() : this._disableResize();
            }, this);
            if (this.settings.get('resizable')) {
                this._enableResize();
            }
        },

        _enableResize: function() {
            if (this._canEnableResize()) {
                this.map.$el.resizable({autoHide: true, handles: "s", stop: this._onResizeStop.bind(this)});
                // workaround until jquery ui is updated
                this.map.$el.on('mouseup', this._onResizeMouseup);
            }
        },
        
        _disableResize: function() {
            if (this._canEnableResize()) {
                this.map.$el.resizable('destroy');
                this.map.$el.off('mouseup', this._onResizeMouseup);
            }
        },
        
        // NOTE: Bound to 'this' by createMap
        _onResizeMouseup: function(event) {
            $(this).width("100%");
        },

        _onResizeStop: function(event, ui) {
            $(event.target).width("100%");
            this._currentHeight = parseInt(this.settings.get('height'), 10);
            this.$msg.height(this._currentHeight).css('overflow', 'hidden');
        },
        
        _canEnableResize: function() {
            // Disable resizing for safari 3 and below only
            return (!($.browser.safari && $.browser.version < "526"));
        },

        _onManagerChange: function(managers, manager) {
            if (this.manager) {
                this.manager.off(null, null, this);
                this.manager = null;
            }
            if (this.resultsModel) {
                this.resultsModel.off(null, null, this);
                this.resultsModel = null;
            }

            this._clearResults();

            if (!manager) {
                this.message("no-search");
                return;
            }

            this.manager = manager;
            this.manager.on("search:start", this._onSearchStart, this);
            this.manager.on("search:progress", this._onSearchProgress, this);
            this.manager.on("search:cancelled", this._onSearchCancelled, this);
            this.manager.on("search:fail", this._onSearchFail, this);
            this.manager.on("search:error", this._onSearchError, this);

            this.resultsModel = this.manager.data(this.settings.get("data"), {
                serialfetch: true,
                output_mode: "json_cols",
                offset: 0,
                count: this.map ? this.map.getMaxClusters() : 100,
                show_empty_fields: "True",
                search: this.map ? this.map.getPostProcessSearch() : ''
            });
            this.resultsModel.on("data", this._onDataUpdate, this);
            this.resultsModel.on("error", this._onSearchError, this);

            this.manager.replayLastSearchEvent(this);
        },

        _onSearchStart: function() {
            this._clearResults();
        },

        _onSearchProgress: function(properties) {
            // No action require on progress at the moment
        },

        _onSearchCancelled: function() {
            this.message('cancelled');
        },

        _onSearchFail: function(state) {
            var msg = Messages.getSearchFailureMessage(state);
            this.message({
                level: "error",
                icon: "warning-sign",
                message: msg
            });
        },

        _onSearchError: function(message, err) {
            var msg = Messages.getSearchErrorMessage(err) || message;
            this.message({
                level: "error",
                icon: "warning-sign",
                message: msg
            });
        },

        _onDataUpdate: function() {
            if (this.resultsModel.hasData()) {
                this.$msg.detach();
                this.$map.css("visibility", "");
                this.results.set(this.resultsModel.data());
            } else {
                this._clearResults();
            }
            this.render();
        },

        _onBoundsChanged: function(){
            if (this.resultsModel) {
                this.resultsModel.set('search', this.map.getPostProcessSearch());
            }
        },

        _onMapClicked: function(e) {
            if (!this.settings.get('drilldown')) {
                return;
            }

            e = e || {};

            // Set up the handlers for default drilldown behavior and preventing
            // the default behavior
            var preventRedirect = false;
            var defaultDrilldown = _.once(_.bind(this._doDrilldown, this, e, this.manager));
            var preventDefault = function() {
                preventRedirect = true;
            };

            e.preventDefault = preventDefault;
            e.drilldown = defaultDrilldown;

            this.trigger('drilldown click click:marker', e, this);

            // Now that the event is trigged, if there is a default action of
            // redirecting, we will execute it (depending on whether the user
            // executed preventDefault()).
            if (this.settings.get("drilldownRedirect") && !preventRedirect) {
                defaultDrilldown();
            }
        },

        _doDrilldown: function(event, manager) {
            if (!event || !manager)
                return;

            var data = event.data;
            if (!data || !data._geo_lat_field || !data._geo_long_field ||
                !data._geo_bounds_south || !data._geo_bounds_west || !data._geo_bounds_north || !data._geo_bounds_east)
                return;

            var options = {
                _geo_lat_field: data._geo_lat_field,
                _geo_lon_field: data._geo_long_field,
                _geo_bounds_south: data._geo_bounds_south,
                _geo_bounds_west: data._geo_bounds_west,
                _geo_bounds_north: data._geo_bounds_north,
                _geo_bounds_east: data._geo_bounds_east
            };

            if (event.altKey)
                options.negate = "true";

            var earliest, latest;
            if (manager.job) {
                earliest = manager.job.properties().searchEarliestTime;
                latest = manager.job.properties().searchLatestTime;
                if (!earliest && manager.job.properties().earliestTime){
                    earliest = util.getEpochTimeFromISO(manager.job.properties().earliestTime);
                }
                if (!latest && manager.job.properties().latestTime){
                    latest = util.getEpochTimeFromISO(manager.job.properties().latestTime);
                }
            }

            this._parseIntentions(manager.query.resolve(), "geoviz", options).done(function(searchString) {
                var params = { 
                    q: searchString,
                    earliest: earliest || '',
                    latest: latest || ''
                };

                var pageInfo = utils.getPageInfo();
                var url = route.search(pageInfo.root, pageInfo.locale, pageInfo.app, { data: params });
                utils.redirect(url, event.ctrlKey);
            });
        },

        _parseIntentions: function(searchString, action, options) {
            var dfd = $.Deferred();

            var params = {};
            for (var o in options) {
                if (options.hasOwnProperty(o))
                    params[o] = options[o];
            }
            params.q = SplunkUtil.addLeadingSearchCommand(searchString);
            params.action = action;
            params.output_mode = "json";

            $.ajax({
                url: SplunkUtil.make_url("/splunkd/__raw/services/search/intentionsparser"),
                data: params,
                type: "get",
                dataType: "json",
                async: true,
                success: function(data) {
                    dfd.resolve(SplunkUtil.stripLeadingSearchCommand(data.fullSearch));
                },
                error: function(xhr, status, error) {
                    dfd.reject(error);
                }
            });

            return dfd.promise();
        },

        _clearResults: function() {
            this.results.set({
                columns: [],
                fields: []
            });
        },

        message: function(info) {
            // Cannot use .hide() on map container because it screws up initial map rendering.
            // Map container dimensions must always evaluate to non-zero.
            // Using visibility hidden instead.
            this.$map.css("visibility", "hidden");
            this.$msg.detach();
            Messages.render(info, this.$msg);
            this.$msg.prependTo(this.$el);
        },

        render: function() {
            this.$el.css({ 'overflow': 'hidden', 'position': 'relative' });
            this.$msg.height(this._currentHeight).css({ 'overflow': 'hidden', 'position': 'absolute', 'width': '100%' });
            return this;
        },

        remove: function() {
            this.destroyMap();
            if(this.settings) {
                this.settings.off();
                if(this.settings._sync) {
                    this.settings._sync.destroy();
                }
            }
            BaseSplunkView.prototype.remove.call(this);
        }
    });

    return SplunkMapView;
});
