/**
 * @author jszeto
 * @date 5/10/13
 *
 * Represents the parameters to pass to the EAI REST endpoint to get sorting, pagination and filtering
 *
 * app {String} - only return objects for this app. Set to "-" to return objects for all apps
 * owner {String} - only return objects for this owner. Set to "-" to return objects for all owners
 * visible {Boolean} - if true, return the visible objects
 * nameFilter {String} - search string to match against the object's name. Can contain wildcards
 * count {Number} - number of objects to return
 * offset {Number} - index of first object to return
 * sort_dir {Enum} - (asc/desc) return objects in ascending or descending order of the sort_key
 * sort_key {String} - field to use for sorting
 * search {String} - default is "". If specified, is the expression used to filter the response. Otherwise,
 * use getCalculatedSearch to return the search filter string.
 */

define(
    [
        'jquery',
        'underscore',
        'backbone',
        'models/Base',
        'splunk.util'
    ],
    function($, _, Backbone, BaseModel, splunk_utils) {
        return BaseModel.extend({
            initialize: function(attributes, options) {
                BaseModel.prototype.initialize.apply(this, arguments);
            },

            /**
             * Generates the search string to pass in the EAI Request. This is calculated from the app, owner, visibleInApp and
             * filter attributes.
             */
            getCalculatedSearch: function() {
                var app = this.get("app");
                var owner = this.get("owner");
                var nameFilter = this.get("nameFilter");
                var visible = splunk_utils.normalizeBoolean(this.get("visible"));

                var searchString = "";
                if (!_.isUndefined(owner)) {
                    searchString += '(eai:acl.owner="' + owner + '")';
                }

                if (!_.isUndefined(app) && !visible) {
                    if (!_.isEmpty(searchString))
                        searchString += ' AND ';
                    searchString += '(eai:acl.app="' + app + '")';
                }

                if (!_.isUndefined(nameFilter) && !_.isEmpty(nameFilter)) {
                    if (!_.isEmpty(searchString))
                        searchString += ' AND ';
                    searchString += nameFilter;
                }

                return searchString;
            }

        });
    }
);

