define(function(require) {
    var _ = require('underscore');
    var $ = require('jquery');
    var BaseInput = require('./base');
    var TextInputView = require('../../textinputview');

    var TextInput = BaseInput.extend({
        initialize: function() {
            var inputOptions = _.defaults({
                el: $('<div class="splunk-view"></div>').appendTo(this.el),
                id: _.uniqueId(this.id + '-input')
            }, this.options);
            this.textbox = new TextInputView(inputOptions);
            this.textbox.on('change', this.handleChange, this);
            
            // Always use the inner view's settings, so we don't have two of them
            this.options.settings = this.textbox.settings;
            BaseInput.prototype.initialize.call(this, {inputId: this.textbox.getInputId()});
            
            // Special events for text box
            this.delegateEvents({
                'change input': 'handleChange',
                'keyup input': 'checkEnterKey'
            });
        },
        getValue: function() {
            return this.textbox.val();
        },
        setValue: function(v) {
            this.textbox.val(v);
        },
        render: function() {
            this.renderLabel();
            this.textbox.render();
            return this;
        },
        _getWrappedView: function() {
            return this.textbox;
        },
        
        checkEnterKey: function(e) {
            if (e.keyCode === 13) {
                this.submit();
            }
        }
    });
    
    return TextInput;
});