define([
    'underscore',
    'jquery',
    'backbone',
    'module',
    'views/Base',
    'views/shared/controls/SyntheticCheckboxControl'
    ],
    function(
        _,
        $,
        Backbone,
        module,
        BaseView,
        SyntheticCheckboxControl
    ) {
    return BaseView.extend({
        moduleId: module.id,
        className: 'push-margins',
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.children.read = new BaseView();
            this.children.write = new BaseView();

            //listeners
            this.model.perms.on('change:Everyone.read', function() {
                this.toggleEveryone('read');
            }, this);
            this.model.perms.on('change:Everyone.write', function() {
                this.toggleEveryone('write');
            }, this);
        },
        appendRow: function(roleName, className) {
            this.$('tbody').append(
                '<tr class="'+ roleName + '">\
                    <td class="role-name">' + roleName + '</td>\
                    <td class="perms-read ' + roleName + '-checkbox"></td>\
                    <td class="perms-write ' + roleName + '-checkbox"></td>\
                </tr>'
            );
            this.children.readCheckbox = new SyntheticCheckboxControl({
                modelAttribute: roleName +'.read',
                model: this.model.perms,
                checkboxClassName: className + " " + roleName + " read btn"
            });
            this.children.writeCheckbox = new SyntheticCheckboxControl({
                modelAttribute: roleName + '.write',
                model: this.model.perms,
                checkboxClassName: className + " " + roleName + " write btn"
            });

            this.$('td.perms-read.'+ roleName + '-checkbox').append(this.children.readCheckbox.render().el);
            this.$('td.perms-write.'+ roleName + '-checkbox').append(this.children.writeCheckbox.render().el);

            this.children.read.children[roleName ] = this.children.readCheckbox;
            this.children.write.children[roleName] = this.children.writeCheckbox;
        },
        toggleEveryone: function(col) {
            var everyoneChecked = this.model.perms.get('Everyone.' + col),
                view = this.children[col];
            _.each(view.children, function(checkbox, role) {
                if (role !== 'Everyone') {
                    if (everyoneChecked) {
                        checkbox.disable();
                    } else {
                        checkbox.enable();
                    }
                }
            });
        },
        render: function() {
            this.$el.html(this.compiledTemplate({
                _: _
            }));

            this.appendRow(_("Everyone").t());

            this.collection.each(function(roleModel) {
                this.appendRow(roleModel.entry.get("name"), 'role');
            }.bind(this));

            this.toggleEveryone('read');
            this.toggleEveryone('write');

            return this;
        },
        template: '\
            <table class="table table-striped table-condensed table-scroll table-border-row">\
                <thead>\
                    <tr>\
                        <td></td>\
                        <th class="perms-read"><%- _("Read").t() %></th>\
                        <th class="perms-write"><%- _("Write").t() %></th>\
                    </tr>\
                </thead>\
                <tbody>\
                </tbody>\
            </table>\
        '
    });
});
