define(
    [
        'underscore',
        'module',
        'views/Base',
        'splunk.util',
        'helpers/user_agent'
    ],
    function(
        _,
        module,
        BaseView,
        util,
        user_agent
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'thead',
            /**
             * @param {Object} options {
             *     model: <models.services.SavedSearch>,
             *     collection: <Backbone.Collection>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.collection.intersectedFields.on('reset', this.render,this);
                this.model.entry.content.on('change:display.events.rowNumbers change:display.events.table.sortDirection change:display.events.table.sortColumn', this.render, this);
            },
            events: {
                'click th': function(e) {
                    var $target = $(e.currentTarget);
                    this.model.entry.content.set({
                        'display.events.table.sortDirection': ($target.hasClass('asc') ? 'desc' : 'asc'),
                        'display.events.table.sortColumn': $target.attr('data-name')
                    });
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                    collection: this.collection.intersectedFields,
                    hasRowNum: util.normalizeBoolean(this.model.entry.content.get('display.events.rowNumbers')),
                    allowRowExpand: this.options.allowRowExpand,
                    content: this.model.entry.content,
                    reorderableHandle: this.options.selectableFields ? 'on': 'off',
                    isRealTime: this.options.isRealTime,
                    is_ie7: (user_agent.isIE7()) ? 'ie7': '',
                    _: _
                }));
                return this;
            },
            template: '\
                <tr class="">\
                    <% if (allowRowExpand) { %>\
                        <th class="col-info"><i class="icon-info"></i></th>\
                    <% } %>\
                    <% if(hasRowNum) { %>\
                        <th class="line-num <%- is_ie7 %>">&nbsp;</th>\
                    <% }%>\
                    <th class="col-time <%- content.get("display.events.table.sortColumn") ? "sorts" : "" %> <%- is_ie7 %>">_time</th>\
                    <% collection.each(function(model) { %>\
                        <% var active = (!isRealTime && (content.get("display.events.table.sortColumn") == model.get("name"))) ? "active": ""%>\
                        <% var dir = (!isRealTime && (active==="active")) ? content.get("display.events.table.sortDirection") : ""%>\
                        <% var sorts = (!isRealTime) ? "sorts" : ""; %>\
                        <th class=" reorderable <%- sorts %> <%-active%> <%-dir%>" data-name="<%- model.get("name") %>"><span class="reorderable-label <%- reorderableHandle %>"><%- _(model.get("name")).t() %></span></th>\
                    <% }) %>\
                </tr>\
            '
        });
    }
);

