define(
    [
        'underscore',
        'jquery',
        'module',
        'views/shared/PopTart',
        'views/shared/vizcontrols/components/Master'
    ],
    function(_, $, module, PopTart, Component){
        return PopTart.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         report: <models.Report>,
             *         state: <models.SplunkDBaseV2>
             *     },
             *     saveOnApply: <Boolean>
             *   }
             */

            options: {
                saveOnApply: false
            },

            initialize: function(options) {
                PopTart.prototype.initialize.apply(this, arguments);
                this.children.visualizationControls && this.children.visualizationControls.remove();
                this.children.visualizationControls = new Component({
                    model: this.model.visualization
                });
                $(window).on('keydown.' + this.cid, this.windowKeydown.bind(this));
            },
            events: {
                'click .viz-editor-apply': function(e) {
                    this.model.visualization.validate();
                    if(this.model.visualization.isValid()) {
                        this.model.report.entry.content.set($.extend({}, this.model.visualization.toJSON()));
                        if(this.options.saveOnApply) {
                            this.model.report.save();
                        }
                        this.hide();
                    }
                    e.preventDefault();
                },
                'click .viz-editor-cancel': function(e) {
                    this.hide();
                    e.preventDefault();
                }
            },
            windowKeydown: function (e) {
                var escapeKeyCode = 27,
                    enterKeyCode = 13;
                
                if (e.keyCode == escapeKeyCode)  {
                    $('.viz-editor-cancel').click(); 
                }
                
                if (e.keyCode == enterKeyCode)  {
                    $('.viz-editor-apply').click(); 
                }
            },
            remove: function() {
                $(window).off('keydown.' + this.cid);
                PopTart.prototype.remove.apply(this, arguments);
                return this;
            },
            render: function() {
                this.$el.html(PopTart.prototype.template);
                this.$el.append(this.template);
                this.$('.popdown-dialog-body').append(this.children.visualizationControls.render().el);
                // ghetto hack to override default padding on poptart ;_;
                this.$('.popdown-dialog-body').removeClass('popdown-dialog-padded');
                return this;
            },
            template: '\
                    <a class="viz-editor-cancel btn pull-left">'+_("Cancel").t()+'</a>\
                    <a class="viz-editor-apply btn btn-primary pull-right"> '+_("Apply").t()+'</a>\
            '
        });
    }
);
