define(
    [
        'underscore',
        'jquery',
        'module',
        'views/shared/PopTart'
    ],
    function(_, $, module, PopTart){
        return PopTart.extend({
            moduleId: module.id,
            className: 'popdown-dialog color-picker-container',
            initialize: function() {
                PopTart.prototype.initialize.apply(this, arguments);
                this.clone = this.model.clone();
            },
            events: {
                'click .swatch': function(e) {
                    var hashPrefixedColor = $(e.currentTarget).data().color,
                        hexColor = '0x'+hashPrefixedColor.substring(1);
                        
                    this.clone.set({ 'color': hexColor });
                    this.$('.swatch-hex input').val(hashPrefixedColor.substring(1));
                    this.$('.big-swatch').css('background-color', hashPrefixedColor);
                    e.preventDefault();
                },
                'click .color-picker-apply': function(e) {
                    this.model.set({
                        'color': this.clone.get('color'),
                        'shadedcolor': this.options.shadeColor(this.clone.get('color').substring(2), -40)
                    });
                    this.model.trigger('color-picker-apply', this.options.index);
                    this.hide();
                    e.stopPropagation();
                },
                'click .color-picker-cancel': function(e) {
                    this.hide();
                    e.stopPropagation();
                },
                'keyup .hex-input': function(e) {
                    var colorStr = $(e.currentTarget).val();
                    
                    this.clone.set('color', '0x'+colorStr);
                    this.$('.big-swatch').css('background-color', '#'+colorStr);
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(PopTart.prototype.template);
                this.$el.append(this._buttonTemplate);
                this.$('.popdown-dialog-body').addClass('color-picker-content');
                var $rangePickerContent = $('<div class="clearfix"></div>').appendTo(this.$('.popdown-dialog-body'));
                $rangePickerContent.append(this.compiledTemplate({
                    model: this.clone
                }));
            },
            template: '\
                <div class="swatches">\
                    <ul class="swatch-holder unstyled">\
                        <li class="swatch" data-color="#7e9f44" style="background-color: #7e9f44"></li>\
                        <li class="swatch" data-color="#ebe42d" style="background-color: #ebe42d"></li>\
                        <li class="swatch" data-color="#d13b3b" style="background-color: #d13b3b"></li>\
                        <li class="swatch" data-color="#6cb8ca" style="background-color: #6cb8ca"></li>\
                        <li class="swatch" data-color="#f7912c" style="background-color: #f7912c"></li>\
                        <li class="swatch" data-color="#956e96" style="background-color: #956e96"></li>\
                        <li class="swatch" data-color="#c2da8a" style="background-color: #c2da8a"></li>\
                        <li class="swatch" data-color="#fac61d" style="background-color: #fac61d"></li>\
                        <li class="swatch" data-color="#ebb7d0" style="background-color: #ebb7d0"></li>\
                        <li class="swatch" data-color="#324969" style="background-color: #324969"></li>\
                        <li class="swatch" data-color="#d85e3d" style="background-color: #d85e3d"></li>\
                        <li class="swatch" data-color="#a04558" style="background-color: #a04558"></li>\
                    </ul>\
                </div>\
                <div class="big-swatch" data-color="<%- model.get("color").substring(2) %>" style="background-color: #<%- model.get("color").substring(2) %>;"></div>\
                <div class="swatch-hex">\
                    <div class="input-prepend views-shared-controls-textcontrol">\
                        <span class="add-on">#</span>\
                        <input type="text" class="hex-input" value="<%-model.get("color").substring(2) %>">\
                    </div>\
                </div>\
            ',
            _buttonTemplate: '\
                <div class="color-picker-buttons clearfix">\
                    <a class="color-picker-cancel btn pull-left">'+_("Cancel").t()+'</a>\
                    <a class="color-picker-apply btn btn-primary pull-right"> '+_("Apply").t()+'</a>\
                </div>\
            '
        });
    }
);
